% Parameters and moments
clear
clc

% Load GMM estimates
load('../GMM/gmm_data')

% addpath('C:\Users\degroot\Dropbox\DRT-Finance\Code\Toolbox')

% Set IES
P.psi = 2.5;

% Options
ntaskids = 500;
nreps = 1000;
TSs = {'noTS','noTS'};
prefs = {'current','revised'};
nicknames = {'C_noTS','R_noTS'};
Fnestmoms = [15,15];
Fnmoments = 29;
nmods = numel(TSs);
% params = {'$\gamma$','$\beta$','$\rho_a$','$\sigma_a$',...
%           '$\mu_y$','$\mu_d$','$\sigma_y$'14,'$\psi_d$','$\pi_{dy}$','$J$','pval'};
params = {'gamma';'beta';'rhoa';'siga';'muy';'mud';'sigy';'psid';'pidy';'J';'pval'};   
paramord = [1,2,5,3,4,6];
Fnparam = numel(params)-2;

PsiDSE = sqrt(diag(OmegaD));
Vlin = variables;
for imod = 1:nmods
    % Load parameters, obj. func. values, and moments
    fnprefix = ['../../Fortran/Baseline/rslt/',...
        prefs{imod},'-',TSs{imod},'-psi' num2str(P.psi,'%.1f')];
    file = dir([fnprefix '-Phis.txt']);
    disp([file.name, ' ', file.date])
    Phis = reshape(textread([fnprefix,'-Phis.txt'],'%f'),[ntaskids,Fnparam]);
    Js = textread([fnprefix '-Js.txt'],'%f');
    Psis = reshape(textread([fnprefix,'-Psis.txt'],'%f'),[ntaskids,Fnmoments]);

    % Means, SDs, and percentiles
    %   Parameters
    meanPhis = mean(Phis);
    SDPhis = std(Phis);
    perPhis = quantile(Phis,[.05,.95]);
    %   Obj. func. values
    Js = nreps*Js/(1+nreps);
    meanJs = mean(Js);
    SDJs = std(Js);
    perJs = quantile(Js,[.05,.95]);
    Jpvals = chi2cdf(Js,Fnestmoms(imod) - Fnparam,'upper');
    meanJpvals = mean(Jpvals);
    SDJpvals = std(Jpvals);
    perJpvals = quantile(Jpvals,[.05,.95]);
    %   Phi p-values
    meanPsis = mean(Psis);
    tstats = (Psis - repmat(PsiD',[ntaskids,1]))./repmat(PsiDSE',[ntaskids,1]);
    meantstat = (meanPsis - PsiD')./PsiDSE';
    Phipvals = 2*tcdf(-abs(tstats),ntaskids-1);
    meanpval = mean(Phipvals);
    rejper = 100*sum(Phipvals<.05,1)/ntaskids;
    
%     disp('---------------------------------------------------------------------------------------------------')
%     disp(['    ', models{imod}, ' estimates'])
%     disp('---------------------------------------------------------------------------------------------------')
     
    % Table of parameter estimates
    tab{1,imod} = table([meanPhis,meanJs,meanJpvals]',[SDPhis,SDJs,SDJpvals]',...
            [perPhis(1,:),perJs(1),perJpvals(1)]',[perPhis(2,:),perJs(2),perJpvals(2)]');
    tab{1,imod}.Properties.VariableNames = ...
        strcat(nicknames{imod},{'_mean','_SD','p5','p95'});
    tab{1,imod}.Properties.RowNames = params;
%     format shortg; disp(tab{1,imod})

    % Table of moments
    tab{2,imod} = table(PsiD,PsiDSE,meanPsis',meantstat');
    tab{2,imod}.Properties.VariableNames = ...
        ['data','data_SE',strcat(nicknames{imod},{'_mean','_tstat'})];
    tab{2,imod}.Properties.RowNames = M.desc;
%     format shortg; disp(tab{2,imod})

    % Use for Decomposition
    % Estimated parameters 
    for iparam = 1:Fnparam
        eval(['P.' params{iparam,1} ' = meanPhis(iparam);']);
    end
    P.theta = (1-P.gamma)/(1-1/P.psi);

    % Set preference dependent parameters
    if strcmp(prefs{imod},'current')
        P.omega = 0;
    elseif strcmp(prefs{imod},'revised')
        P.omega = P.beta;
    end

    % Use linear solution as guess
    Slin = steadystate(P);
    [Tmat,~,C,eu] = linmodel(P,Slin,Vlin);

    % Campbell-Shiller solution
    if all(eu == 1)
        csguess = [...
            Slin.kappay0,Slin.kappay1,C(Vlin.zy),Tmat(Vlin.zy,Vlin.a),...
            Slin.kappad0,Slin.kappad1,C(Vlin.zd),Tmat(Vlin.zd,Vlin.a)]';    
        [S,R] = solution_cs(P,csguess);
        if ~R.flag
            disp('Campbell-Shiller approx. not found')
            return;
        end
    else
        disp('Log-linear solution does not exist and/or is not unique')
        return;
    end

    % from Mathematica
%     decRfIN = 1200*((-1).*log(P.beta).*P.theta+P.muy.*P.psi.^(-1).*P.theta+(-1).*((-1) ...
%     +P.theta).*(P.muy+S.kappay0+S.etay0.*((-1)+S.kappay1)));
%     decRfCF = 1200*(1/2).*(( ...
%       -1).*P.psi.^(-2).*P.sigy^2.*(P.psi+P.theta+(-1).*P.psi.*P.theta) ...
%       .^2);
%     decRfVR = 1200*(1/2).*(((-1).*P.siga^2.*((-1).*P.omega.*P.theta+((-1)+P.theta).* ...
%       S.etay1.*S.kappay1).^2));

    decRfIN = 1200*((1/2).*P.psi.^(-2).*(2.*P.muy.*P.psi+(-2).*log(P.beta).*P.psi.^2));

    decRfCF = 1200*((1/2).*P.psi.^(-2).*P.sigy^2.*((-1).*P.psi.^2+((-1)+P.psi.^2).* ...
      P.theta));

    decRfVR = 1200*((1/2).*P.psi.^(-2).*P.siga^2.*((-1).*P.psi.^2.* ...
      S.etay1.^2.*S.kappay1.^2+P.psi.^2.*P.theta.*((-1).*P.omega+ ...
      S.etay1.*S.kappay1).*(P.omega+S.etay1.*S.kappay1)));


    % by hand
%     decRfIN = 1200*(-log(P.beta)+P.muy/P.psi)
%     decRfCF = 1200*(1/2)*P.sigy^2*((1/P.psi-P.gamma)*(1-P.gamma)-P.gamma^2)
%     decRfVR = 1200*(1/2)*P.siga^2*((P.theta-1)*S.etay1^2*S.kappay1^2-P.theta*P.omega^2)

  
    decRfCFper = 100*decRfCF/(decRfCF+decRfVR);
    decRfVRper = 100*decRfVR/(decRfCF+decRfVR);

    compCF = 1200*P.pidy.*P.psi.^(-1).*P.sigy^2.*(P.psi+P.theta+(-1).*P.psi.* ...
      P.theta);
    compVR = 1200*P.siga^2.*S.etad1.*S.kappad1.*(P.omega.*P.theta+S.etay1.* ...
      (S.kappay1+(-1).*P.theta.*S.kappay1));

    compCFper = 100*compCF/(compCF+compVR);
    compVRper = 100*compVR/(compCF+compVR);
    
    disp('-----------------------------------------')
    disp(['    Model: ' prefs{imod}  ])
    disp( '                 rf          ep'  )
    disp('-----------------------------------------')
    disp(['    CFR Cont.   ' num2str(decRfCFper)  '      ' num2str(compCFper)])
    disp(['     VR Cont.   ' num2str(decRfVRper)  '      ' num2str(compVRper)])

    
    disp(S.etad1/(1-P.omega*P.rhoa))
end

disp('---------------------------------------------------------------------------------------------------')
disp('    Parameter estimates')
disp('---------------------------------------------------------------------------------------------------')
format shortg; 
disp([tab{1,1}(:,[1,2]),tab{1,2}(:,[1,2])])

disp('---------------------------------------------------------------------------------------------------')
disp('    Moments estimates')
disp('---------------------------------------------------------------------------------------------------')
format bank; 
disp([tab{2,1}(:,[1,2]),tab{2,1}(:,[3,4]),tab{2,2}(:,[3,4])])