% Test objective function
clear; clc;
savename = mfilename;
warning('off')

% Save shocks to test Fortran code
saveshocks = 1;

% Set IES
P.psi = 2.5;

% Use specified parameters, comment out to use values in F.guess
% Current
% Phi = [...
%    1.349043673201514
%    0.997883334896484
%    0.997314126848774
%    0.000278437266379
%    0.001573522487415
%    0.000974929895137
%    0.005790804282323
%    1.516734360645215
%    0.815898510532301];

% % Revised
% Phi = [...
%   99.340548905373424
%    0.996406179317914
%    0.989256636610824
%    0.036611707211246
%    0.001650464730542
%    0.001628608148935
%    0.005545688321804
%    1.138177478396462
%    0.62345949848297];

% Preferences: current or revised
O.pref = 'revised';

% Load GMM estimates
load('../GMM/gmm_data')

% Parameters
F.guess = {...  
    'gamma',	5,      0.5,      1,      1000;     % RA       
    'beta',     0.9975, 0.001,    1e-8,   0.9995;   % Discount factor   
    'rhoa',     0.975,  0.01,     1e-8,   0.9995;   % AR preference shock 
    'siga',     0.0005, 0.00025,  1e-8,   1;        % SD preference shock
    'muy',      0.002,  0.00025,  1e-8,   1;        % Mean endowment growth
    'mud',      0.002,  0.00025,  1e-8,   1;        % Mean dividend growth
    'sigy',     0.005,  0.0025,   1e-8,   1;        % SD endowment growth shock
    'psid',     3,      1,        0,      100;      % SD dividend growth
    'pidy',     0,      0.5,      -100,   100};     % Corr. endowment and dividend growth 
F.nparam = size(F.guess,1);

% Empirical targets
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd];

% GMM estimates
PsiD = PsiD(O.smmidx);
OmegaD = OmegaD(O.smmidx,O.smmidx);

% Parameterization and shocks
if ~exist('Phi')
	Phi = cell2mat(F.guess(:,2));
end
mtstream = RandStream('mt19937ar','seed',123456);
RandStream.setGlobalStream(mtstream);
nreps = 1000;
epsy = randn(ndatapers,nreps);
epsd = randn(ndatapers,nreps);
epsa = randn(ndatapers,nreps);
a0 = randn(1,nreps);
if saveshocks
    epsytemp = epsy(:);
    epsdtemp = epsd(:);
    epsatemp = epsa(:);
    a0temp = a0(:);
    save('../../Fortran/Baseline/test/epsy.txt','epsytemp','-ASCII','-double')
    save('../../Fortran/Baseline/test/epsd.txt','epsdtemp','-ASCII','-double')
    save('../../Fortran/Baseline/test/epsa.txt','epsatemp','-ASCII','-double')
    save('../../Fortran/Baseline/test/a0.txt','a0temp','-ASCII','-double')
end

% Objective function
V = variables_cs;
Vlin = variables;

% Objective function
[J,PsiM] = objfunc(...
    Phi,PsiD,OmegaD,M,O,P,F,V,Vlin,...
    ndatapers,nreps,...
    epsy,epsd,epsa,a0,1);
disp(['J =    ', num2str(J)])
