function sims = simulation_cs(P,S,V,epsy,epsd,epsa,a0)

%----------------------------------------------------------------------
% Initialize Simulation
%----------------------------------------------------------------------
[npers,nsims] = size(epsy);
sims = zeros(npers,V.nvar,nsims);
% Shocks
epsyTemp = permute(epsy,[1 3 2]);
epsdTemp = permute(epsd,[1 3 2]);
%----------------------------------------------------------------------
% Simulate time series
%----------------------------------------------------------------------
% Endowment growth 
sims(:,V.Dy,:) = P.muy+P.sigy*epsyTemp(:,1,:);
% Dividend growth 
sims(:,V.Dd,:) = P.mud+P.pidy*P.sigy*epsyTemp(:,1,:)+P.psid*P.sigy*epsdTemp(:,1,:);
% Preference shock
sims(:,V.a,:) = filter(1,[1,-P.rhoa],P.siga*epsa,P.SDa*a0);
% Price-dividend ratio 
sims(:,V.zd,:) = S.etad0+S.etad1*sims(:,V.a,:);
% Risk free return (from ../Coefficients/baseline.nb)
sims(:,V.rf,:) = ...
    (-1).*log(P.beta).*P.theta+P.muy.*P.psi.^(-1).*P.theta+((-1)+ ...
    P.omega.*P.rhoa).*P.theta.*sims(:,V.a,:)+(-1).*((-1)+P.theta).*( ...
    P.muy+S.kappay0+S.etay0.*((-1)+S.kappay1)+S.etay1.*sims(:,V.a,:).* ...
    ((-1)+P.rhoa.*S.kappay1))+(1/2).*((-1).*P.psi.^(-2).*P.sigy^2.*( ...
    P.psi+P.theta+(-1).*P.psi.*P.theta).^2+(-1).*P.siga^2.*((-1).* ...
    P.omega.*P.theta+((-1)+P.theta).*S.etay1.*S.kappay1).^2);
% Long-term rates
n5 = 60; n20 = 240;
m0 = P.theta*log(P.beta) - P.gamma*P.muy + (P.theta-1)*(S.kappay0 + S.etay0*(S.kappay1 - 1));
m1 = 1 - P.omega*P.rhoa;
m2 = (P.theta-1)*S.kappay1*S.etay1-P.theta*P.omega;
m3 = -P.gamma;
Prhoaj = zeros(n20,1);
for j = 0:n20-1
    Prhoaj(j+1) = P.rhoa^j;
end
%	5-year
nestsum = 0;
for k = 2:n5
    nestsum = nestsum + (m2 + m1*sum(Prhoaj(1:n5-k+1)))^2*P.siga^2/2;
end
p5(:,1,:) = n5*m0 + m1*sum(Prhoaj(1:n5))*sims(:,V.a,:) + ...
    + nestsum + m2^2*P.siga^2/2 + n5*m3^2*P.sigy^2/2;
sims(:,V.rf5,:) = -p5(:,1,:)/n5;
%	20-year
nestsum = 0;
for k = 2:n20
    nestsum = nestsum + (m2 + m1*sum(Prhoaj(1:n20-k+1)))^2*P.siga^2/2;
end
p20(:,1,:) = n20*m0 + m1*sum(Prhoaj(1:n20))*sims(:,V.a,:) + ...
    + nestsum + m2^2*P.siga^2/2 + n20*m3^2*P.sigy^2/2;
sims(:,V.rf20,:) = -p20(:,1,:)/n20;