function [draws,Js] = smmalg(...
            PsiD,OmegaD,M,O,F,V,Vlin,...
            ndatapers,nreps,...
            epsy,epsd,epsa,a0,...
            ndraws,Phi0,covmat,...
            c0,c,temperature,resumeSMM,iSMM)

% Number of parameters and bounds
lbounds = cell2mat(F.guess(:,4));
ubounds = cell2mat(F.guess(:,5));

% SMM algorithm
savename = ['Estimation/' O.pref '-' num2str(iSMM)];
if ~resumeSMM
    idraw0 = 0;
    draws = zeros(ndraws,F.nparam);
    Js = zeros(ndraws,1);
    lastJ = 0;
else
	load(savename,'idraw','draws','Js')
    draws(ndraws+1:end,:) = [];
    idraw = min(idraw,ndraws);
    idraw0 = idraw + 1;
    lastdraw = draws(idraw,:)';
    lastJ = Js(idraw);
end
tic
for idraw = idraw0:ndraws
    Jexist = 0;
    while ~Jexist
        % Draw from multivariate normal
        if idraw == 0
            draw = mvnrnd(Phi0,c0^2.*covmat)';
        else
            draw = lastdraw + mvnrnd(zeros(size(Phi0)),c^2.*covmat)';
        end
        
        if all(draw > lbounds) && all(draw < ubounds)
            % J-stat
            J = objfunc(draw,PsiD,OmegaD,M,O,F,V,Vlin,...
                ndatapers,nreps,epsy,epsd,epsa,a0,0);
            Jexist = isreal(J) && ~isinf(J) && ~isnan(J) && J < 1e12;
        end
    end

    % Accept or reject draw
    candJ = -J;
    if idraw == 0
        lastdraw = draw;
        lastJ = candJ;
    else       
        if min(exp((candJ - lastJ)/temperature),1) > rand
%             disp('Draw accepted')
%             disp(draw)
            lastdraw = draw;               
            lastJ = candJ;
        else
%             disp('Draw rejected')
        end
        % Store draw and log-likelihoods
        draws(idraw,:) = lastdraw;
        Js(idraw) = lastJ;
    end
    
    % Display and save
    if mod(idraw,10) == 1 %&& idraw > 0
        toc
        dispme = ['Draw ' num2str(idraw) ': -J = ' num2str(lastJ)];
        if idraw <= 100
            disp(dispme)
        else
            disp([dispme ' Accept %: ' num2str(sum(abs(diff(Js(idraw-100:idraw)))>0))])
        end
        save(savename,'idraw','draws','Js','F')
    end
end