function [S,R,x] = solution_cs(P,x0)

    % Solve
    [x,flag] = csolve('coefficients',x0,[],1e-8,30,P);
    %   Endowment claim coefficients 
    S.kappay0 = x(1);
    S.kappay1 = x(2);
    S.etay0 = x(3);
    S.etay1 = x(4);
    %   Dividend claim coefficients    
    S.kappad0 = x(5);
    S.kappad1 = x(6);
    S.etad0 = x(7);
    S.etad1 = x(8);

    % Flag convergence and calculate unconditional moments
    if flag==0
        R.flag = 1;
        % Expected risk-free return (from ../Coefficients/baseline.nb)
        R.Erf = 100*12*(...
            (-1).*log(P.beta).*P.theta+P.muy.*P.psi.^(-1).*P.theta+(-1).*((-1) ...
            +P.theta).*(P.muy+S.kappay0+S.etay0.*((-1)+S.kappay1))+(1/2).*(( ...
            -1).*P.psi.^(-2).*P.sigy^2.*(P.psi+P.theta+(-1).*P.psi.*P.theta) ...
            .^2+(-1).*P.siga^2.*((-1).*P.omega.*P.theta+((-1)+P.theta).* ...
            S.etay1.*S.kappay1).^2)...
            );
        % Expected endowment and equity return
        R.Ery = 100*12*(S.kappay0+(S.kappay1-1)*S.etay0+P.muy);
        R.Erd = 100*12*(S.kappad0+(S.kappad1-1)*S.etad0+P.mud);
        % Expected equity premium
        R.Eep = 100*12*(P.theta*P.omega + (1-P.theta)*S.kappay1*S.etay1)*S.kappad1*S.etad1*P.siga^2 ...
              + P.gamma*P.pidy*P.sigy^2;         
        % Expected price-dividend ratio 
        R.Epd = S.etad0-log(12);
    else
        R.flag = 0;
    end
end