function [J,PsiM,S,R] = objfunc(...
    Phi,PsiD,OmegaD,M,O,P,F,V,Vlin,...
    nburnsim,T,nreps,epsy,epsd,epsa,a0,epsx,x0,epssigy,sigy0,...
    display) 

% Objective function and moments to return if error
J = 1e12;
PsiM = NaN(numel(O.smmidx),1);

% Check that parameters are in bounds
if  any(Phi < cell2mat(F.guess(:,4))) || ...
    any(Phi > cell2mat(F.guess(:,5)))
    disp('Parameter(s) out of bounds')
    return;
end

% Estimated parameters 
for iparam = 1:F.nparam
    eval(['P.' F.guess{iparam,1} ' = Phi(iparam);']);
end
    
% Implied parameters
P.theta = (1-P.gamma)/(1-1/P.psi);
P.SDa = sqrt(P.siga^2/(1-P.rhoa^2));
P.SDx = sqrt((P.psix*P.sigy)^2/(1-P.rhox^2));
P.SDsigy = sqrt(P.nuy^2/(1-P.rhosigy^2));

% Set preference dependent parameters
if strcmp(O.pref,'current')
    P.omega = 0;
elseif strcmp(O.pref,'revised')
    P.omega = P.beta;
    % Check restriction on preference shock
    %   a^R < 1/beta
    %   log(a) = log(a^R)/(1-beta)
    %       -> a^R = exp((1-beta)*log(a)) < 1/beta
    %   SD(log(a)) = sqrt(siga^2/(1-rhoa^2))
    if P.beta*exp((1-P.beta)*4*sqrt(P.siga^2/(1-P.rhoa^2))) > 1
        disp('Preference parameters violate utility function restriction')
        return;
    end
end

% Use linear solution as guess
Slin = steadystate(P);
if (exp(Slin.ry)-exp(P.muy)<=0) || (exp(Slin.rd)-exp(P.mud)<=0) 
    disp('Parameters yield invalid steady states')
    return;
end
[Tmat,~,C,eu] = linmodel(P,Slin,Vlin);

% Campbell-Shiller solution
if all(eu == 1)
    csguess = [...
        Slin.kappay0,Slin.kappay1,C(Vlin.zy),Tmat(Vlin.zy,[Vlin.a,Vlin.x]),0,...
        Slin.kappad0,Slin.kappad1,C(Vlin.zd),Tmat(Vlin.zd,[Vlin.a,Vlin.x]),0]';    
    [S,R] = solution_cs(P,csguess);
    if ~R.flag
        disp('Campbell-Shiller approx. not found')
        return;
    end
else
    disp('Log-linear solution does not exist and/or is not unique')
    return;
end

% Simulate Campbell-Shiller model
sims_cs = simulation_cs(P,S,V,epsy,epsd,epsa,a0,epsx,x0,epssigy,sigy0,nburnsim);
% Calculate moments
nburn = 2;
%   Dividends (level and price)
d = squeeze(cumprod(exp(sims_cs(:,V.Dd,:))));
d_reshape = reshape(d,[12,T/12,nreps]);
pd = exp(squeeze(sims_cs(:,V.zd,:))).*d;
pd_reshape = reshape(pd,[12,T/12,nreps]);
%   Consumption and dividend growth
g_an = squeeze(sum(reshape(sims_cs(:,[V.Dy,V.Dd],:),[12,T/12,2,nreps])));
gc = squeeze(100*g_an(nburn+1:end,1,:));
gclag = squeeze(100*g_an(nburn:end-1,1,:));
gd = squeeze(100*g_an(nburn+1:end,2,:));
gdlag = squeeze(100*g_an(nburn:end-1,2,:));
%   Price-dividend ratio
d_an = squeeze(sum(d_reshape));
pd_an = squeeze(pd_reshape(end,:,:));
zd = log(pd_an(nburn+1:end,:)./d_an(nburn+1:end,:));
zdlag = log(pd_an(nburn:end-1,:)./d_an(nburn:end-1,:));
%   Returns
rf_an = squeeze(sum(reshape(sims_cs(:,V.rf,:),[12,T/12,nreps])));
rftemp = 100*rf_an(2:end,:);
rf = rftemp(nburn:end,:);
rflag = rftemp(nburn-1:end-1,:);
rdtemp = 100*log((pd_an(2:end,:)+d_an(2:end,:))./pd_an(1:end-1,:));
rd = rdtemp(nburn:end,:);
rdlag = rdtemp(nburn-1:end-1,:);
ep = rd-rf;
rf5_an = squeeze(sum(reshape(sims_cs(:,V.rf5,:),[12,T/12,nreps])));
rf5 = 100*rf5_an(nburn+1:end,:);
rf20_an = squeeze(sum(reshape(sims_cs(:,V.rf20,:),[12,T/12,nreps])));
rf20 = 100*rf20_an(nburn+1:end,:);
%	Means and Standard deviations
means = [mean(gc);mean(gd);mean(rd);mean(rf);mean(zd);mean(rf5);mean(rf20)];
sds = [std(gc);std(gd);std(rd);std(rf);std(zd)];
%   Auto-correlations
corrs(1,:) = diag(corr(gc,gclag));
corrs(2,:) = diag(corr(gd,gdlag));
corrs(3,:) = diag(corr(rd,rdlag));
corrs(4,:) = diag(corr(rf,rflag));
corrs(5,:) = diag(corr(zd,zdlag));
%	Cross-Correlations
corrs(6,:) = diag(corr(gc,gd)); 
corrs(7,:) = diag(corr(gc,rd));  
corrs(8,:) = diag(corr(gd,rd));
%   Predictability correlations
corrs(9,:) = diag(corr(ep,zdlag));
corrs(10,:) = diag(corr(gc,zdlag));

%   Medians
PsiM([M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20]) = mean(means,2);
PsiM([M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd]) = mean(sds,2);
PsiM([...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag]) = mean(corrs,2);

% Calculate objective function value
PsiM = PsiM(O.smmidx);
Psidiff = PsiM - PsiD;
J = (Psidiff'/OmegaD)*Psidiff;
if display
    disp([  'J: ' num2str(J) ...
            ' RA: ' num2str(Phi(1)) ...
            ' IES: ' num2str(Phi(2))])    
end