function [T,M,C,eu] = linmodel(P,S,V)

% [T,M,C,eu] = linmodel(P,S,V)
%   Solves the log-linear model with GENSYS
% Inputs:
%     P     :   Structure of parameters
%     S     :   Structure of steady state values
%     V     :   Structure of variable locations and names
% Output:
%     T     :   Transition matrix
%     M     :   Impact matrix
%     C     :   Constant matrix
%     eu    :   [existence, uniqueness]

%---------------------------------------------
%   Initialize GENSYS components
%---------------------------------------------
G0 = zeros(V.nvar);
G1 = zeros(V.nvar);
Psi = zeros(V.nvar,V.nshock);
Pi = zeros(V.nvar,V.nfore);
CC = zeros(V.nvar,1);
j = 0;
%---------------------------------------------
j=j+1;  % [Consumption Euler equation]
%---------------------------------------------
G0(j,V.eDy) = P.theta*(1-1/P.psi);
G0(j,V.ea) = -P.theta*P.omega;
G0(j,V.ezy) = P.theta*S.kappay1;
G0(j,V.a) = P.theta;
G0(j,V.zy) = -P.theta;
CC(j) = -P.theta*(log(P.beta)+S.kappay0);
%---------------------------------------------
j=j+1;  % [Dividend Euler equation]
%---------------------------------------------
G0(j,V.eDd) = 1;
G0(j,V.eDy) = P.theta*(1-1/P.psi)-1;
G0(j,V.ea) = -P.theta*P.omega;
G0(j,V.ezd) = S.kappad1;
G0(j,V.ezy) = (P.theta-1)*S.kappay1;
G0(j,V.a) = P.theta;
G0(j,V.zd) = -1;
G0(j,V.zy) = 1-P.theta;
CC(j) = -P.theta*log(P.beta)+(1-P.theta)*S.kappay0-S.kappad0;
%---------------------------------------------
j=j+1;  % [Endowment] 
%---------------------------------------------
G0(j,V.Dy) = 1;
G1(j,V.x) = 1;
CC(j) = P.muy;
Psi(j,V.epsy) = P.sigy;
Psi(j,V.epsa) = P.piya*P.siga;
%---------------------------------------------
j=j+1;  % [Dividend] 
%---------------------------------------------
G0(j,V.Dd) = 1;
G1(j,V.x) = P.phid;
CC(j) = P.mud;
Psi(j,V.epsd) = P.psid*P.sigy;
Psi(j,V.epsa) = P.pida*P.siga;
%---------------------------------------------
j=j+1;  % [Preference Shock Process] 
%---------------------------------------------
G0(j,V.a) = 1;
G1(j,V.a) = P.rhoa;
Psi(j,V.epsa) = P.siga;
%---------------------------------------------
j=j+1;  % [Long-Run Risk Shock Process] 
%---------------------------------------------
G0(j,V.x) = 1;
G1(j,V.x) = P.rhox;
Psi(j,V.epsx) = P.psix*P.sigy;
 
%---------------------------------------------
j=j+1;  %   [FE a] 
%---------------------------------------------
G0(j,V.a) = 1;
G1(j,V.ea) = 1;
Pi(j,V.fea) = 1;
%---------------------------------------------
j=j+1;  %   [FE Dy] 
%---------------------------------------------
G0(j,V.Dy) = 1;
G1(j,V.eDy) = 1;
Pi(j,V.feDy) = 1;
%---------------------------------------------
j=j+1;  %   [FE zy] 
%---------------------------------------------
G0(j,V.zy) = 1;
G1(j,V.ezy) = 1;
Pi(j,V.fezy) = 1;
%---------------------------------------------
j=j+1;  %   [FE Dd] 
%---------------------------------------------
G0(j,V.Dd) = 1;
G1(j,V.eDd) = 1;
Pi(j,V.feDd) = 1;
%---------------------------------------------
j=j+1;  %   [FE zd] 
%---------------------------------------------
G0(j,V.zd) = 1;
G1(j,V.ezd) = 1;
Pi(j,V.fezd) = 1;

%---------------------------------------------
%   Solve Linear Model
%---------------------------------------------
[T,C,M,~,~,~,~,eu] = gensys(G0,G1,CC,Psi,Pi);        