% Parameters and moments
clear
clc

% Load GMM estimates
load('../../../GMM_GD_TS/gmm_data')

% Set IES
Ppsi = 2.5;

ntaskids = 160;
nreps = 1000;

models = {'revised'};
nicknames = {'R'};
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag];
Fnestmom = numel(O.smmidx);
params = {'\gamma','\beta','\rho_a','\sigma_a', ...
          '\muy','mud','sigy',...
          'phid','rhox','psix','piya','pida','J'};
Fnparam = numel(params)-1;

PsiD = PsiD(O.smmidx);
PsiDSE = sqrt(diag(OmegaD(O.smmidx,O.smmidx)));
moms = M.desc(O.smmidx);

for imod = 1:numel(models)
    % Load parameters, obj. func. values, and moments
    fnprefix = ['../Results_GD_nopsid_fixies/' models{imod} '/psi' num2str(Ppsi) '-'];
    Phis = reshape(textread([fnprefix 'Phis.txt'],'%f'),[ntaskids,Fnparam]);
    Js = textread([fnprefix 'Js.txt'],'%f');
    Psis = reshape(textread([fnprefix 'Psis.txt'],'%f'),[ntaskids,Fnestmom]);

    % Means, SDs, and percentiles
    %   Parameters
    meanPhis = mean(Phis);
    SDPhis = std(Phis);
    perPhis = quantile(Phis,[.05,.95]);
    %   Obj. func. values
    Js = nreps*Js/(1+nreps);
    meanJs = mean(Js);
    SDJs = std(Js);
    perJs = quantile(Js,[.05,.95],2);
    %   T-stats
    meanPsis = mean(Psis);
    tstats = (Psis - repmat(PsiD',[ntaskids,1]))./repmat(PsiDSE',[ntaskids,1]);
    pvals = 2*tcdf(-abs(tstats),ntaskids-1);
    meanpval = mean(pvals);
    rejper = 100*sum(pvals<.05,1)/ntaskids;
    
    disp('---------------------------------------------------------------------------------------------------')
    disp(['    ', models{imod}, ' estimates'])
    disp('---------------------------------------------------------------------------------------------------')
     
    % Table of parameter estimates
    tab{1,imod} = table([meanPhis,meanJs]',[SDPhis,SDJs]',...
            [perPhis(1,:),perJs(1)]',[perPhis(2,:),perJs(2)]');
    tab{1,imod}.Properties.VariableNames = ...
        strcat(nicknames{imod},{'_mean','_SD','p5','p95'});
    tab{1,imod}.Properties.RowNames = params;
    format shortg; disp(tab{1,imod})

    % Table of moments
    tab{2,imod} = table(PsiD,PsiDSE,meanPsis',meanpval',rejper');
    tab{2,imod}.Properties.VariableNames = ...
        ['data','data_SE',strcat(nicknames{imod},{'_mean','_pval','_RejPer'})];
    tab{2,imod}.Properties.RowNames = moms;
    format shortg; disp(tab{2,imod})
end

disp('---------------------------------------------------------------------------------------------------')
disp('    Parameter estimates')
disp('---------------------------------------------------------------------------------------------------')
format shortg; disp(tab{1,1}(:,[1,2]))

disp('---------------------------------------------------------------------------------------------------')
disp('    Moments estimates')
disp('---------------------------------------------------------------------------------------------------')
format bank; disp(tab{2,1}(:,[1,2,3]))

% %% Create tables for LaTeX
% 
% % -------------------------------------------------------------------------------------
% %    Parameter estimates
% % -------------------------------------------------------------------------------------
% savename = 'parameters_fixcf.tex';
% ncols = 6;
% 
% % Header
% header = cell(4,ncols);
% % header{1,1} = '\begin{table}[!htb]\footnotesize';
% % header{2,1} = '\centering';
% header{1,1} = '\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}cccccc}';
% header{2,1} = '\toprule';
% header{3,1} = 'Parameter & Current & Revised & Parameter & Current & Revised \\';
% header{4,1} = '\midrule';
% 
% % Entries
% nformat = {'%.2f','%.2f','%.4f','%.5f','%.4f','%.2f'};
% entries = [];
% entry = [];
% for ipar = 1:Fnparam+1
%     iparord = paramord(ipar);
%     % Column 1
%     entry{1,1} = ['$',params{iparord},'$'];
%     % Columns 2-3
%     for imod = 1:numel(models)
%         % Row 1: mean
%         entry{1,1+imod} = ['$',...
%             num2str(table2array(tab{1,imod}(iparord,1)),nformat{iparord}),'$'];
%         % Row 2: percentiles
%         entry{2,1+imod} = ['\scs$(',...
%             num2str(table2array(tab{1,imod}(iparord,3)),nformat{iparord}),',',...
%             num2str(table2array(tab{1,imod}(iparord,4)),nformat{iparord}),')$'];
%     end
%     % Concatenate entries
%     entries = [entries;entry];
% end
% % Split entries in half
% nrows = size(entries,1);
% entries = [entries(1:nrows/2,:),entries(nrows/2+1:end,:)];
% 
% % Add line breaks, cell breaks, and spacing
% entries(:,end) = strcat(entries(:,end),' \\ ');
% entries(:,1:end-1) = strcat(entries(:,1:end-1),' & ');
% entries(1:2:end,end) = strcat(entries(1:2:end,end),' [-4pt] ');
% entries(2,end) = strcat(entries(2,end),'\rule{0pt}{\normalbaselineskip}\linebreak');
% entries(4,end) = strcat(entries(4,end),'\cline{4-6}\rule{0pt}{\normalbaselineskip}\linebreak');
% 
% % Footer
% footer = cell(2,ncols);
% footer{1,1} = '\bottomrule';
% footer{2,1} = '\end{tabular*}';
% % footer{3,1} = '\caption{Parameter estimates. Current: $J=21.081$ $(20.925,21.122)$; Revised: $J=21.084$ $(20.925,21.114)$.}';
% % footer{3,1} = '\label{tab:baseline param}';
% % footer{4,1} = '\end{table}\normalsize';
% 
% % Concatenate 
% tablatex = [header;entries;footer];
% 
% % Save as *.tex
% fid = fopen(savename,'w');
% [nrows,ncols] = size(tablatex);
% tabformat = [repmat('%s ',1,ncols),' \n'];
% for row = 1:nrows
%     fprintf(fid,tabformat,tablatex{row,:});
% end
% fclose(fid);
% 
% % -------------------------------------------------------------------------------------
% %    Moments
% % -------------------------------------------------------------------------------------
% savename = 'moments_fixcf.tex';
% moms = {'E[r_d]','E[r_f]','E[z_d]',...
%     'SD[r_d]','SD[r_f]','SD[z_d]',...
%     'Corr[r_f,r_{f,-1}]','Corr[z_d,z_{d,-1}]'};
% ncols = 10;
% 
% % Header
% header = cell(6,ncols);
% % header{1,1} = '\begin{table}[!htb]\footnotesize';
% % header{2,1} = '\centering';
% header{1,1} = '\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}l*{4}{S[table-format=1.2]}l*{4}{S[table-format=1.2]}}';
% header{2,1} = '\toprule';
% header{3,1} = ' & \multicolumn{2}{c}{Data} & \multicolumn{2}{c}{Model} &  & \multicolumn{2}{c}{Data} & \multicolumn{2}{c}{Model} \\';
% header{4,1} = ' \cline{2-3} \cline{4-5}  \cline{7-8} \cline{9-10} \rule{0pt}{\normalbaselineskip}\linebreak';
% header{5,1} = '{Moment} & {Mean} & {SE} & {Current} & {Revised} & {Moment} & {Data} & {SE} & {Current} & {Revised}\\';
% header{6,1} = '\midrule';
% 
% % Entries
% nformat = cell(1,Fnestmom);
% nformat(:) = {'%.2f'};
% entries = [];
% entry = [];
% for imom = 1:Fnestmom
%     imomord = momord(imom);
%     % Column 1
%     entry{1,1} = ['$',moms{imomord},'$'];
%     % Column 2: Data
%     entry{1,2} = ...
%         num2str(table2array(tab{2,1}(imomord,1)),nformat{imomord});
%     % Column 3: SE
%     entry{1,3} = ...
%         num2str(table2array(tab{2,1}(imomord,2)),nformat{imomord});
%     % Columns 4-5: Model mean
%     for imod = 1:numel(models)
%         entry{1,3+imod} = ...
%             num2str(table2array(tab{2,imod}(imomord,3)),nformat{imomord});
%     end
%     % Concatenate entries
%     entries = [entries;entry];
% end
% % Split entries in half
% % dashes = cell(1,5);
% % dashes(:) = {'{$-$}'};
% % entries = [entries;dashes];
% nrows = size(entries,1);
% entries = [entries(1:nrows/2,:),entries(nrows/2+1:end,:)];
% 
% % Add line breaks, cell breaks, and spacing
% entries(:,end) = strcat(entries(:,end),' \\ ');
% entries(:,1:end-1) = strcat(entries(:,1:end-1),' & ');
% 
% % Footer
% footer = cell(2,ncols);
% footer{1,1} = '\bottomrule';
% footer{2,1} = '\end{tabular*}';
% % footer{3,1} = '\caption{Unconditional short-sample moments given the mean parameter estimates.}';
% % footer{4,1} = '\label{tab:baseline mom}';
% % footer{5,1} = '\end{table}\normalsize';
% 
% % Concatenate 
% tablatex = [header;entries;footer];
% 
% % Save as *.tex
% fid = fopen(savename,'w');
% [nrows,ncols] = size(tablatex);
% tabformat = [repmat('%s ',1,ncols),' \n'];
% for row = 1:nrows
%     fprintf(fid,tabformat,tablatex{row,:});
% end
% fclose(fid);