% Test objective function
clear; clc;
savename = mfilename;
warning('off')

% Save shocks to test Fortran code
saveshocks = 1;

% Set IES
P.psi = 2.5;

% Set dividend shock
P.psid = 0;

% Use specified parameters, comment out to use values in F.guess
% Phi = [...
%         2.63167715023919
%         0.99948932856965
%         0.956694150599695
%         0.0179233040800372
%         0.00152128278568631
%         0.0014302419293434
%         0.00406561440421657
%         2.65543854328832
%         0.99839406036329
%         0.0278212416312588
%         -0.0573481809128269
%         -1.03874754835385];

% Preferences: current or revised
O.pref = 'revised';

% Load GMM estimates
load('../GMM/gmm_data')

% Parameters
F.guess = {...  
    'gamma',	5,      0.5,      1,      1000;     % RA       
    'beta',     0.9975, 0.001,    1e-8,   0.9995;   % Discount factor   
    'rhoa',     0.975,  0.01,     1e-8,   0.9995;   % AR preference shock 
    'siga',     0.0005, 0.00025,  1e-8,   1;        % SD preference shock
    'muy',      0.002,  0.00025,  1e-8,   1;        % Mean endowment growth
    'mud',      0.002,  0.00025,  1e-8,   1;        % Mean dividend growth
    'sigy',     0.005,  0.0025,   1e-8,   1;        % SD endowment growth shock
    'phid',     4,      1,        1e-8,   100;      % Long-run corr. endowment and dividend growth
    'rhox',     0.98,   0.05,     1e-8,   0.9995;   % Long-run risk persistence
    'psix',     0.05,   0.025,    1e-8,   0.2;      % SD long-run risk shock
    'piya',     0,      0.5,      -100,   100;      % Corr. endowment growth and preference shock      
    'pida',     0,      0.5,      -100,   100};     % Corr. dividend growth and preference shock 
F.nparam = size(F.guess,1);

% Empirical targets
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag];

% GMM estimates
PsiD = PsiD(O.smmidx);
OmegaD = OmegaD(O.smmidx,O.smmidx);

% Parameterization and shocks
if ~exist('Phi')
	Phi = cell2mat(F.guess(:,2));
end
mtstream = RandStream('mt19937ar','seed',123456);
RandStream.setGlobalStream(mtstream);
nreps = 1000;
epsy = randn(ndatapers,nreps);
epsd = randn(ndatapers,nreps);
epsa = randn(ndatapers,nreps);
a0 = randn(1,nreps);
epsx = randn(ndatapers,nreps);
x0 = randn(1,nreps);
if saveshocks
    epsytemp = epsy(:);
    epsdtemp = epsd(:);
    epsatemp = epsa(:);
    a0temp = a0(:);
    epsxtemp = epsx(:);
    x0temp = x0(:);
    save('../../Fortran/Extended-Demand/test/epsy.txt','epsytemp','-ASCII','-double')
    save('../../Fortran/Extended-Demand/test/epsd.txt','epsdtemp','-ASCII','-double')
    save('../../Fortran/Extended-Demand/test/epsa.txt','epsatemp','-ASCII','-double')
    save('../../Fortran/Extended-Demand/test/a0.txt','a0temp','-ASCII','-double')
    save('../../Fortran/Extended-Demand/test/epsx.txt','epsxtemp','-ASCII','-double')
    save('../../Fortran/Extended-Demand/test/x0.txt','x0temp','-ASCII','-double')
end

% Objective function
V = variables_cs;
Vlin = variables;

% Objective function
[J,PsiM] = objfunc(...
    Phi,PsiD,OmegaD,M,O,P,F,V,Vlin,...
    ndatapers,nreps,...
    epsy,epsd,epsa,a0,epsx,x0,1);
disp(['J =    ', num2str(J)])