function sims = simulation_cs(P,S,V,epsy,epsd,epsa,a0,epsx,x0)

%----------------------------------------------------------------------
% Initialize Simulation
%----------------------------------------------------------------------
[npers,nsims] = size(epsy);
sims = zeros(npers,V.nvar,nsims);
% Shocks
epsyTemp = permute(epsy,[1 3 2]);
epsdTemp = permute(epsd,[1 3 2]);
epsaTemp = permute(epsa,[1 3 2]);
%----------------------------------------------------------------------
% Simulate time series
%----------------------------------------------------------------------
% Long-run risk
sims(:,V.x,:) = filter(P.psix*P.sigy,[1,-P.rhox],epsx,P.SDx*x0);
% Endowment growth 
sims(:,V.Dy,:) = P.muy+sims(:,V.x,:)+...
    P.sigy*epsyTemp(:,1,:)+ ...
    P.piya*P.siga*epsaTemp(:,1,:);
% Dividend growth
sims(:,V.Dd,:) = P.mud+P.phid*sims(:,V.x,:)+ ...
    P.psid*P.sigy*epsdTemp(:,1,:)+...
    P.pida*P.siga*epsaTemp(:,1,:);
% Preference shock
sims(:,V.a,:) = filter(P.siga,[1,-P.rhoa],epsa,P.SDa*a0);
% Price-dividend ratio
sims(:,V.zd,:) = S.etad0+S.etad1*sims(:,V.a,:)+S.etad2*sims(:,V.x,:);
% Risk free return (from ../Coefficients/...)
sims(:,V.rf,:) = ...
    (-1).*log(P.beta).*P.theta+((-1)+P.omega.*P.rhoa).*P.theta.*sims( ...
    :,V.a,:)+P.psi.^(-1).*P.theta.*(P.muy+sims(:,V.x,:))+(-1).*((-1)+ ...
    P.theta).*(P.muy+(-1).*S.etay1.*sims(:,V.a,:)+sims(:,V.x,:)+ ...
    S.kappay0+S.etay0.*((-1)+S.kappay1)+P.rhoa.*S.etay1.*sims(:,V.a,:) ...
    .*S.kappay1+S.etay2.*sims(:,V.x,:).*((-1)+P.rhox.*S.kappay1))+( ...
    1/2).*((-1).*P.psi.^(-2).*P.sigy^2.*(P.psi+P.theta+(-1).*P.psi.* ...
    P.theta).^2+(-1).*P.psix.^2.*P.sigy^2.*((-1)+P.theta).^2.* ...
    S.etay2.^2.*S.kappay1.^2+(-1).*P.psi.^(-2).*P.siga^2.*(P.psi.*( ...
    P.piya+S.etay1.*S.kappay1)+P.theta.*(P.piya+(-1).*P.piya.*P.psi+ ...
    P.psi.*(P.omega+(-1).*S.etay1.*S.kappay1))).^2);
% Long-term rates
n5 = 60; n20 = 240;
m0 = P.theta*log(P.beta) - P.gamma*P.muy + (P.theta-1)*(S.kappay0 + S.etay0*(S.kappay1 - 1));
m1 = 1 - P.omega*P.rhoa;
m2 = (P.theta-1)*S.kappay1*S.etay1-P.theta*P.omega - P.gamma*P.piya;
m3 = -P.gamma;
m4 = -1/P.psi;
m5 = (P.theta-1)*S.kappay1*S.etay2;
Prhoaj = zeros(n20,1);
Prhoxj = zeros(n20,1);
for j = 0:n20-1
    Prhoaj(j+1) = P.rhoa^j;
    Prhoxj(j+1) = P.rhox^j;
end
%	5-year
nestsuma = 0;
nestsumx = 0;
for k = 2:n5
    nestsuma = nestsuma + (m2 + m1*sum(Prhoaj(1:n5-k+1)))^2*P.siga^2/2;
    nestsumx = nestsumx + (m5 + m4*sum(Prhoxj(1:n5-k+1)))^2*(P.psix*P.sigy)^2/2;
end
p5(:,1,:) = n5*m0 ...
    + m1*sum(Prhoaj(1:n5))*sims(:,V.a,:) + m4*sum(Prhoxj(1:n5))*sims(:,V.x,:) ...
    + nestsuma + m2^2*P.siga^2/2 + n5*m3^2*P.sigy^2/2 ...
    + nestsumx + m5^2*(P.psix*P.sigy)^2/2;
sims(:,V.rf5,:) = -p5(:,1,:)/n5;
%	20-year
nestsuma = 0;
nestsumx = 0;
for k = 2:n20
    nestsuma = nestsuma + (m2 + m1*sum(Prhoaj(1:n20-k+1)))^2*P.siga^2/2;
    nestsumx = nestsumx + (m5 + m4*sum(Prhoxj(1:n20-k+1)))^2*(P.psix*P.sigy)^2/2;
end
p20(:,1,:) = n20*m0 ...
    + m1*sum(Prhoaj(1:n20))*sims(:,V.a,:) + m4*sum(Prhoxj(1:n20))*sims(:,V.x,:) ...
    + nestsuma + m2^2*P.siga^2/2 + n20*m3^2*P.sigy^2/2 ...
    + nestsumx + m5^2*(P.psix*P.sigy)^2/2;
sims(:,V.rf20,:) = -p20(:,1,:)/n20;