% Parameters and moments
clear
clc

% Risk-free rate moments (1: include; 0: do not include)
rfmom = 0;

% Remove corr(gc,rd) and corr(gd,rd) (1: yes; 0: no)
nocorr = 1;

% Set IES
Ppsi = 2.5;

% Load GMM estimates
load('../../../GMM_GD/gmm_data')

ntaskids = 160;
nreps = 1000;

% Empirical targets
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag];
if ~rfmom
    O.smmidx = O.smmidx([1:8,10:13,15:end]);
    if nocorr
        O.smmidx = O.smmidx([1:14,17:18]);
    end
end

Fnestmom = numel(O.smmidx);
params = {'\gamma','\beta', ...
          '\muy','mud','sigy','psid','pidy',...
          'phid','rhox','psix','rhosigy','nuy','J'};
Fnparam = numel(params)-1;

PsiD = PsiD(O.smmidx);
PsiDSE = sqrt(diag(OmegaD(O.smmidx,O.smmidx)));
moms = M.desc(O.smmidx);

% Load parameters, obj. func. values, and moments
fnprefix = 'Results_noVR_SV_GD_fixies';
if ~rfmom
    fnprefix = [fnprefix '_norf'];
    if nocorr
        fnprefix = ['Additional_Specifications/' fnprefix '_nocorr'];
    end        
end
fnprefix = ['../' fnprefix '/psi' num2str(Ppsi,'%.1f') '-'];
Phis = reshape(textread([fnprefix 'Phis.txt'],'%f'),[ntaskids,Fnparam]);
Js = textread([fnprefix 'Js.txt'],'%f');
Psis = reshape(textread([fnprefix 'Psis.txt'],'%f'),[ntaskids,Fnestmom]);

% Means, SDs, and percentiles
%   Parameters
meanPhis = mean(Phis);
SDPhis = std(Phis);
perPhis = quantile(Phis,[.05,.95]);
%   Obj. func. values
Js = nreps*Js/(1+nreps);
meanJs = mean(Js);
SDJs = std(Js);
perJs = quantile(Js,[.05,.95]);
%   T-stats
meanPsis = mean(Psis);
tstats = (Psis - repmat(PsiD',[ntaskids,1]))./repmat(PsiDSE',[ntaskids,1]);
pvals = 2*tcdf(-abs(tstats),ntaskids-1);
meanpval = mean(pvals);
rejper = 100*sum(pvals<.05,1)/ntaskids;

% disp('---------------------------------------------------------------------------------------------------')
% disp('NO VR estimates')
% disp('---------------------------------------------------------------------------------------------------')

% Table of parameter estimates
tab{1} = table([meanPhis,meanJs]',[SDPhis,SDJs]',...
        [perPhis(1,:),perJs(1)]',[perPhis(2,:),perJs(2)]');
tab{1}.Properties.VariableNames = ...
    strcat('noVR',{'_mean','_SD','p5','p95'});
tab{1}.Properties.RowNames = params;
% format shortg; disp(tab{1})

% Table of moments
tab{2} = table(PsiD,PsiDSE,meanPsis',meanpval',rejper');
tab{2}.Properties.VariableNames = ...
    ['data','data_SE',strcat('noVR',{'_mean','_pval','_RejPer'})];
tab{2}.Properties.RowNames = moms;
% format shortg; disp(tab{2})

disp('---------------------------------------------------------------------------------------------------')
disp('    Parameter estimates')
disp('---------------------------------------------------------------------------------------------------')
format shortg; disp(tab{1,1}(:,[1,2]))

disp('---------------------------------------------------------------------------------------------------')
disp('    Moments estimates')
disp('---------------------------------------------------------------------------------------------------')
format bank; disp(tab{2}(:,[1,2,3]))