function sims = simulation_cs(P,S,V,epsy,epsd,epsx,x0,epssigy,sigy0,nburnsim)

%----------------------------------------------------------------------
% Initialize Simulation
%----------------------------------------------------------------------
[npers,nsims] = size(epssigy);
sims = zeros(npers-nburnsim,V.nvar,nsims);
% Shocks
epsyTemp = permute(epsy,[1 3 2]);
epsdTemp = permute(epsd,[1 3 2]);
%----------------------------------------------------------------------
% Simulate time series
%----------------------------------------------------------------------
% Variance endowment shock
sigy2 = zeros(npers,1,nsims);
sigy2(:) = P.sigy^2 + filter(1,[1,-P.rhosigy],P.nuy*epssigy,P.SDsigy*sigy0);
sigy2 = max(0,sigy2);
sims(:,V.sigy2,:) = sigy2(nburnsim+1:end,1,:);
sigy = sqrt(sims(:,V.sigy2,:));
% Long-run risk
x = zeros(npers,1,nsims);
x(:) = filter(1,[1,-P.rhox],...
    P.psix*squeeze(sqrt(sigy2(:,1,:))).*epsx,P.SDx*x0);
sims(:,V.x,:) = x(nburnsim+1:end,1,:);
% Endowment growth 
sims(:,V.Dy,:) = P.muy+sims(:,V.x,:) + sigy.*epsyTemp;
% Dividend growth
sims(:,V.Dd,:) = P.mud+P.phid*sims(:,V.x,:) + ...
    P.pidy*sigy.*epsyTemp + P.psid*sigy.*epsdTemp;
% Price-dividend ratio
sims(:,V.zd,:) = S.etad0 + S.etad1*sims(:,V.x,:) + S.etad2*sims(:,V.sigy2,:);
% Risk free return (from ../Coefficients/baseline.nb)
sims(:,V.rf,:) = ...
    (-1).*log(P.beta).*P.theta+P.psi.^(-1).*P.theta.*(P.muy+sims(:, ...
    V.x,:))+(-1).*((-1)+P.theta).*(P.muy+(-1).*S.etay0+(-1).*S.etay2.* ...
    sims(:,V.sigy2,:)+sims(:,V.x,:)+(-1).*S.etay1.*sims(:,V.x,:)+ ...
    S.kappay0+(S.etay0+S.etay2.*(P.sigy^2+(-1).*P.rhosigy.*P.sigy^2+ ...
    P.rhosigy.*sims(:,V.sigy2,:))+P.rhox.*S.etay1.*sims(:,V.x,:)).* ...
    S.kappay1)+(1/2).*((-1).*P.psi.^(-2).*(P.psi+P.theta+(-1).*P.psi.* ...
    P.theta).^2.*sims(:,V.sigy2,:)+(-1).*P.nuy.^2.*((-1)+P.theta).^2.* ...
    S.etay2.^2.*S.kappay1.^2+(-1).*P.psix.^2.*((-1)+P.theta).^2.* ...
    S.etay1.^2.*sims(:,V.sigy2,:).*S.kappay1.^2);
% Long-term rates
n5 = 60; n20 = 240;
m0 = P.theta*log(P.beta) - P.gamma*P.muy ...
    + (P.theta-1)*(S.kappay0 + S.etay0*(S.kappay1 - 1)) ...
    + (P.theta-1)*S.kappay1*S.etay2*(1-P.rhosigy)*P.sigy^2;
m3 = -P.gamma;
m4 = -1/P.psi;
m5 = (P.theta-1)*S.kappay1*S.etay1;
m6 = (P.theta-1)*S.etay2*(S.kappay1*P.rhosigy-1);
m7 = (P.theta-1)*S.kappay1*S.etay2;
Prhoxj = zeros(n20,1);
Prhosigyj = zeros(n20,1);
for j = 0:n20-1
    Prhoxj(j+1) = P.rhox^j;
    Prhosigyj(j+1) = P.rhosigy^j;
end
%	5-year
nestsumm6m3 = zeros(n5,1);
nestsumm5m4a = zeros(n5-1,1);
for k = 2:n5
    nestsumm6m3(k-1) = (m6+m3^2/2)*sum(Prhosigyj(1:k-1));
    nestsumm5m4a(k-1) = P.rhosigy^(k-2)*(m5 + m4*sum(Prhoxj(1:n5-k+1)))^2*P.psix^2/2;
end
coefsigy2 = (m6+m3^2/2)*sum(Prhosigyj(1:n5))+sum(nestsumm5m4a(1:n5-1)) + ...
    P.rhosigy^(n5-1)*m5^2*P.psix^2/2;
nestsumnuy2 = 0;
for t = 0:n5-2
    nestsumm5m4b = 0;
    for k = 2:n5-t-1
        nestsumm5m4b = nestsumm5m4b + ...
            P.rhosigy^(k-2)*(m5 + m4*sum(Prhoxj(1:n5-k-t)))^2*P.psix^2/2;
    end
    nestsumnuy2 = nestsumnuy2 + ...
        (m7 + (m6+m3^2/2)*sum(Prhosigyj(1:n5-t-1)) ...
        + nestsumm5m4b + P.rhosigy^(n5-t-2)*m5^2*P.psix^2/2)^2;
end
nestsumsumm5m4 = 0;
for t = 1:n5-2
    for k = 2:n5-t
        nestsumsumm5m4 = nestsumsumm5m4 + ...
            P.rhosigy^(k-2)*(m5 + m4*sum(Prhoxj(1:n5-k-t+1)))^2;
    end
end
p5(:,1,:) = n5*m0 ...
    + m4*sum(Prhoxj(1:n5))*sims(:,V.x,:) ...
    + coefsigy2*sims(:,V.sigy2,:) ...
    + nestsumnuy2*P.nuy^2/2 + m7^2*P.nuy^2/2 ...
    + (sum(nestsumm6m3(1:n5-1)) + nestsumsumm5m4*P.psix^2/2 ...
        + m5^2*P.psix^2*sum(Prhosigyj(1:n5-1))/2)*(1-P.rhosigy)*P.sigy^2;
sims(:,V.rf5,:) = -p5(:,1,:)/n5;
%	20-year
nestsumm6m3 = zeros(n20,1);
nestsumm5m4a = zeros(n20-1,1);
for k = 2:n20
    nestsumm6m3(k-1) = (m6+m3^2/2)*sum(Prhosigyj(1:k-1));
    nestsumm5m4a(k-1) = P.rhosigy^(k-2)*(m5 + m4*sum(Prhoxj(1:n20-k+1)))^2*P.psix^2/2;
end
coefsigy2 = (m6+m3^2/2)*sum(Prhosigyj(1:n20))+sum(nestsumm5m4a(1:n20-1)) + ...
    P.rhosigy^(n20-1)*m5^2*P.psix^2/2;
nestsumnuy2 = 0;
for t = 0:n20-2
    nestsumm5m4b = 0;
    for k = 2:n20-t-1
        nestsumm5m4b = nestsumm5m4b + ...
            P.rhosigy^(k-2)*(m5 + m4*sum(Prhoxj(1:n20-k-t)))^2*P.psix^2/2;
    end
    nestsumnuy2 = nestsumnuy2 + ...
        (m7 + (m6+m3^2/2)*sum(Prhosigyj(1:n20-t-1)) ...
        + nestsumm5m4b + P.rhosigy^(n20-t-2)*m5^2*P.psix^2/2)^2;
end
nestsumsumm5m4 = 0;
for t = 1:n20-2
    for k = 2:n20-t
        nestsumsumm5m4 = nestsumsumm5m4 + ...
            P.rhosigy^(k-2)*(m5 + m4*sum(Prhoxj(1:n20-k-t+1)))^2;
    end
end
p20(:,1,:) = n20*m0 ...
    + m4*sum(Prhoxj(1:n20))*sims(:,V.x,:) ...
    + coefsigy2*sims(:,V.sigy2,:) ...
    + nestsumnuy2*P.nuy^2/2 + m7^2*P.nuy^2/2 ...
    + (sum(nestsumm6m3(1:n20-1)) + nestsumsumm5m4*P.psix^2/2 ...
        + m5^2*P.psix^2*sum(Prhosigyj(1:n20-1))/2)*(1-P.rhosigy)*P.sigy^2;
sims(:,V.rf20,:) = -p20(:,1,:)/n20;