function h = gmmresid(PsiD,data,D,M)

% Data series
nyears = length(data);
gc = data(:,D.gc);
gd = data(:,D.gd);
rd = data(:,D.rd);  
rf = data(:,D.rf); 
zd = data(:,D.zd);
rf5 = data(:,D.rf5); 
rf20 = data(:,D.rf20); 
ep = data(:,D.ep); 
gclag = data(:,D.gclag); 
gdlag = data(:,D.gdlag); 
rdlag = data(:,D.rdlag);
rflag = data(:,D.rflag);
zdlag = data(:,D.zdlag);

% Moments
%   Means
mu_gc = PsiD(M.mu_gc);
mu_gd = PsiD(M.mu_gd);
mu_rd = PsiD(M.mu_rd);
mu_rf = PsiD(M.mu_rf);
mu_zd = PsiD(M.mu_zd);
mu_rf5 = PsiD(M.mu_rf5);
mu_rf20 = PsiD(M.mu_rf20);
%   Standard deviations
sd_gc = PsiD(M.sd_gc);
sd_gd = PsiD(M.sd_gd);
sd_rd = PsiD(M.sd_rd);
sd_rf = PsiD(M.sd_rf);
sd_zd = PsiD(M.sd_zd); 
%   Auto correlations
ac_gc = PsiD(M.ac_gc); 
ac_gd = PsiD(M.ac_gd); 
ac_rd = PsiD(M.ac_rd); 
ac_rf = PsiD(M.ac_rf); 
ac_zd = PsiD(M.ac_zd);
%   Cross correlations
corr_gcgd = PsiD(M.corr_gcgd);
corr_gcrd = PsiD(M.corr_gcrd);
corr_gcrf = PsiD(M.corr_gcrf);
corr_gczd = PsiD(M.corr_gczd);
corr_gdrd = PsiD(M.corr_gdrd);
corr_gdrf = PsiD(M.corr_gdrf);
corr_gdzd = PsiD(M.corr_gdzd);
corr_rdrf = PsiD(M.corr_rdrf);
corr_rdzd = PsiD(M.corr_rdzd);
corr_rfzd = PsiD(M.corr_rfzd);
%   Predictability correlations
corr_ep1zdlag = PsiD(M.corr_ep1zdlag);
corr_gc1zdlag = PsiD(M.corr_gc1zdlag);

% Equity premium moments
mu_ep = mu_rd - mu_rf;
sd_ep = sqrt(sd_rd^2 + sd_rf^2 - 2*corr_rdrf*sd_rd*sd_rf);

% Residual
h = zeros(nyears,M.nmoments);
j=0;
%   Means
j=j+1; h(:,j) = mu_gc - gc;
j=j+1; h(:,j) = mu_gd - gd;
j=j+1; h(:,j) = mu_rd - rd;
j=j+1; h(:,j) = mu_rf - rf;
j=j+1; h(:,j) = mu_zd - zd;
j=j+1; h(:,j) = mu_rf5 - rf5;
j=j+1; h(:,j) = mu_rf20 - rf20;
%   Standard deviations
j=j+1; h(:,j) = sd_gc^2 - (gc - mu_gc).^2;
j=j+1; h(:,j) = sd_gd^2 - (gd - mu_gd).^2;
j=j+1; h(:,j) = sd_rd^2 - (rd - mu_rd).^2;
j=j+1; h(:,j) = sd_rf^2 - (rf - mu_rf).^2;
j=j+1; h(:,j) = sd_zd^2 - (zd - mu_zd).^2;
%   Auto correlations
j=j+1; h(:,j) = ac_gc*sd_gc*sd_gc - (gc - mu_gc).*(gclag - mu_gc); 
j=j+1; h(:,j) = ac_gd*sd_gd*sd_gd - (gd - mu_gd).*(gdlag - mu_gd);
j=j+1; h(:,j) = ac_rd*sd_rd*sd_rd - (rd - mu_rd).*(rdlag - mu_rd); 
j=j+1; h(:,j) = ac_rf*sd_rf*sd_rf - (rf - mu_rf).*(rflag - mu_rf);
j=j+1; h(:,j) = ac_zd*sd_zd*sd_zd - (zd - mu_zd).*(zdlag - mu_zd);
%   Cross correlations
j=j+1; h(:,j) = corr_gcgd*sd_gc*sd_gd - (gc - mu_gc).*(gd - mu_gd);
j=j+1; h(:,j) = corr_gcrd*sd_gc*sd_rd - (gc - mu_gc).*(rd - mu_rd);
j=j+1; h(:,j) = corr_gcrf*sd_gc*sd_rf - (gc - mu_gc).*(rf - mu_rf);
j=j+1; h(:,j) = corr_gczd*sd_gc*sd_zd - (gc - mu_gc).*(zd - mu_zd);
j=j+1; h(:,j) = corr_gdrd*sd_gd*sd_rd - (gd - mu_gd).*(rd - mu_rd);
j=j+1; h(:,j) = corr_gdrf*sd_gd*sd_rf - (gd - mu_gd).*(rf - mu_rf);
j=j+1; h(:,j) = corr_gdzd*sd_gd*sd_zd - (gd - mu_gd).*(zd - mu_zd);
j=j+1; h(:,j) = corr_rdrf*sd_rd*sd_rf - (rd - mu_rd).*(rf - mu_rf);
j=j+1; h(:,j) = corr_rdzd*sd_rd*sd_zd - (rd - mu_rd).*(zd - mu_zd);
j=j+1; h(:,j) = corr_rfzd*sd_rf*sd_zd - (rf - mu_rf).*(zd - mu_zd);
%   Predictability correlations
j=j+1; h(:,j) = corr_ep1zdlag*sd_ep*sd_zd - (ep - mu_ep).*(zdlag - mu_zd);
j=j+1; h(:,j) = corr_gc1zdlag*sd_gc*sd_zd - (gc - mu_gc).*(zdlag - mu_zd);