function [D,M] = indexes

% Indexes: Names moments, data, and assigns indexes
% Inputs:
%     [None]
% Output:
%     D     :   Structure of data variable locations and names
%     M     :   Structure of moment locations and names

% Moments
M.names = {...
      'mu_gc'               %1  Mean consumption growth
      'mu_gd'               %2  Mean dividend growth
      'mu_rd'               %3  Mean equity return
      'mu_rf'               %4  Mean risk-free rate
      'mu_zd'               %5  Mean price-dividend ratio
      'mu_rf5'              %6  Mean 5-year risk-free rate
      'mu_rf20'             %7  Mean 20-year risk-free rate
      'sd_gc'               %8  SD consumption growth
      'sd_gd'               %9  SD dividend growth
      'sd_rd'               %10  SD equity return
      'sd_rf'               %11 SD risk-free rate
      'sd_zd'               %12 SD price-dividend ratio
      'ac_gc'               %13 Autocorr. consumption growth
      'ac_gd'               %14 Autocorr. dividend growth
      'ac_rd'               %15 Autocorr. equity return
      'ac_rf'               %16 Autocorr. risk-free rate
      'ac_zd'               %17 Autocorr. price-dividend ratio
      'corr_gcgd'           %18 Corr. consumption growth and dividend growth
      'corr_gcrd'           %19 Corr. consumption growth and equity return
      'corr_gcrf'           %20 Corr. consumption growth and risk-free rate
      'corr_gczd'           %21 Corr. consumption growth and price-dividend ratio
      'corr_gdrd'           %22 Corr. dividend growth and equity return
      'corr_gdrf'           %23 Corr. dividend growth and risk-free rate
      'corr_gdzd'           %24 Corr. dividend growth and price-dividend ratio
      'corr_rdrf'           %25 Corr. equity return and risk-free rate   
      'corr_rdzd'           %26 Corr. equity return and price-dividend ratio  
      'corr_rfzd'           %27 Corr. risk-free rate and price-dividend ratio         
      'corr_ep1zdlag'       %28 Corr. 1-year equity premium and 1-year lagged price-dividend ratio
      'corr_gc1zdlag'       %29 Corr. 1-year consumption growth and 1-year lagged price-dividend ratio
    };   

% Variables titles        
M.desc = {  'E[\Delta c]'               
            'E[\Delta d]'               
            'E[r_d]'               
            'E[r_f]'               
            'E[z_d]'               
            'E[r_{f,5}]'               
            'E[r_{f,2}0]'               
            'SD[\Delta c]'               
            'SD[\Delta d]'               
            'SD[r_d]'               
            'SD[r_f]'               
            'SD[z_d]'               
            'AC[\Delta c]'               
            'AC[\Delta d]'               
            'AC[r_d]'               
            'AC[r_f]'               
            'AC[z_d]'               
            'Corr[\Delta c,\Delta d]'           
            'Corr[\Delta c,r_d]'           
            'Corr[\Delta c,r_f]'           
            'Corr[\Delta c,z_d]'           
            'Corr[\Delta d,r_d]'           
            'Corr[\Delta d,r_f]'           
            'Corr[\Delta d,z_d]'           
            'Corr[r_d,r_f]'           
            'Corr[r_d,z_d]'           
            'Corr[r_f,z_d]'                 
            'Corr[ep,zd_{d,-1}]'           
            'Corr[\Delta c,zd_{d,-1}]'                     
         };    
    
% Data series   
D.names = { ...
          'gc'          %1  Growth rate of consumption
          'gd'         	%2  Growth rate of dividends  
          'rd'          %3  Return to equities   
          'rf'         	%4  Risk-free rate
          'zd'      	%5  Price-dividend ratio
          'rf5'         %6  5-year risk-free rate
          'rf20'        %7  20-year risk-free rate
          'ep'      	%8  Equity premium
          'gclag'     	%9  Lagged consumption growth
          'gdlag'     	%10  Lagged dividend growth
          'rdlag'     	%11 Lagged equity return
          'rflag'     	%12 Lagged risk-free rate
          'zdlag'     	%13 Lagged price-dividend ratio                   
        };   

% Establish index for moments
M.nmoments = length(M.names);
for j = 1:M.nmoments
   eval(['M.' M.names{j} ' = j;']);      
end

% Establish index for data series
D.ndatavar = length(D.names);
for j = 1:D.ndatavar
   eval(['D.' D.names{j} ' = j;']);      
end