% GMM estimates and standard errors
clear; clc
saving = 'off';
savename = 'gmm_data';

%% Load and transform data (annualized)
[D,M] = indexes;
startyear = 1929;
endyear = 2017;
    
% Load data
year = xlsread('..\..\Data\DRT-data.xlsx','Data','A5:A93');
dataraw = xlsread('..\..\Data\DRT-data.xlsx','Data','B5:H93');
sample = (year >= startyear) & (year <= endyear);
dataraw = dataraw(sample,:);
ndatapers = 12*numel(startyear:endyear);

% Assign to variables
nburn = 2;
nyearssub = ndatapers/12-nburn;
data = zeros(nyearssub,D.ndatavar);
data(:,D.gc) = dataraw(nburn+1:end,1);
data(:,D.gd) = dataraw(nburn+1:end,2);
data(:,D.rd) = dataraw(nburn+1:end,3);
data(:,D.rf) = dataraw(nburn+1:end,4);
data(:,D.zd) = dataraw(nburn+1:end,5);
data(:,D.rf5) = dataraw(nburn+1:end,6);
data(:,D.rf20) = dataraw(nburn+1:end,7);
data(:,D.ep) = dataraw(nburn+1:end,3)-dataraw(nburn+1:end,4);
data(:,D.gclag) = dataraw(nburn:end-1,1); 
data(:,D.gdlag) = dataraw(nburn:end-1,2); 
data(:,D.rdlag) = dataraw(nburn:end-1,3);
data(:,D.rflag) = dataraw(nburn:end-1,4);
data(:,D.zdlag) = dataraw(nburn:end-1,5);

% Initial values
PsiD0 = zeros(M.nmoments,1);
%   Means
PsiD0(M.mu_gc) = mean(data(:,D.gc));
PsiD0(M.mu_gd) = mean(data(:,D.gd));
PsiD0(M.mu_rd) = mean(data(:,D.rd));
PsiD0(M.mu_rf) = mean(data(:,D.rf));
PsiD0(M.mu_zd) = mean(data(:,D.zd));
PsiD0(M.mu_rf5) = mean(data(:,D.rf5));
PsiD0(M.mu_rf20) = mean(data(:,D.rf20));
%   Standard deviations
PsiD0(M.sd_gc) = std(data(:,D.gc),1); 
PsiD0(M.sd_gd) = std(data(:,D.gd),1);
PsiD0(M.sd_rd) = std(data(:,D.rd),1);
PsiD0(M.sd_rf) = std(data(:,D.rf),1);
PsiD0(M.sd_zd) = std(data(:,D.zd),1);
%   Auto correlations
PsiD0(M.ac_gc) = corr(data(:,D.gc),data(:,D.gclag));
PsiD0(M.ac_gd) = corr(data(:,D.gd),data(:,D.gdlag));
PsiD0(M.ac_rd) = corr(data(:,D.rd),data(:,D.rdlag));
PsiD0(M.ac_rf) = corr(data(:,D.rf),data(:,D.rflag));
PsiD0(M.ac_zd) = corr(data(:,D.zd),data(:,D.zdlag));
%   Cross correlations
PsiD0(M.corr_gcgd) = corr(data(:,D.gc),data(:,D.gd));
PsiD0(M.corr_gcrd) = corr(data(:,D.gc),data(:,D.rd));
PsiD0(M.corr_gcrf) = corr(data(:,D.gc),data(:,D.rf));
PsiD0(M.corr_gczd) = corr(data(:,D.gc),data(:,D.zd));
PsiD0(M.corr_gdrd) = corr(data(:,D.gd),data(:,D.rd));
PsiD0(M.corr_gdrf) = corr(data(:,D.gd),data(:,D.rf));
PsiD0(M.corr_gdzd) = corr(data(:,D.gd),data(:,D.zd));
PsiD0(M.corr_rdrf) = corr(data(:,D.rd),data(:,D.rf));
PsiD0(M.corr_rdzd) = corr(data(:,D.rd),data(:,D.zd));
PsiD0(M.corr_rfzd) = corr(data(:,D.rf),data(:,D.zd));
%   Predictability correlations
PsiD0(M.corr_ep1zdlag) = corr(data(:,D.ep),data(:,D.zdlag));
PsiD0(M.corr_gc1zdlag) = corr(data(:,D.gc),data(:,D.zdlag));

%% GMM on data to get PsiD and OmegaD

% Set GMM parameters
%   Sets # of steps over initial weighting matrix
nstep = 2;
% This variable is used to choose the method to calculate covariance matrix
%   0 : No lags used in computing s0
%   1 : Flat window used when computing s0 (may not be psd)
%   2 : Bartlett window used when computing s0
%   3 : Parzen window used when computing s0
wtype = 2;
% Number of lags used in window methods for calculating s0
wlag = 10;
      
% Execute GMM
fmean = @(x) mean(gmmresid(x,data,D,M))';
fresid = @(x) gmmresid(x,data,D,M);
[PsiD,OmegaD] = gmm(...
    PsiD0,fmean,fresid,nstep,...
    wtype,wlag);

% Equity premium
FD = 0*PsiD;
FD([M.mu_rd,M.mu_rf]) = [1,-1];
meanEP = PsiD(M.mu_rd) - PsiD(M.mu_rf); 
seEP = sqrt(FD'*OmegaD*FD); 

% Results
meanGMM = PsiD;
seGMM = sqrt(diag(OmegaD));

%% Display GMM estimates
disp('GMM estimates')
tab = table(PsiD0,meanGMM,seGMM);
tab.Properties.RowNames = M.desc;
format short; disp(tab)

%% Save estimates
if strcmp(saving,'on')
    % Use only diagonal of covariance matrix
    OmegaD = diag(diag(OmegaD));
    save(savename,'ndatapers','M','PsiD','OmegaD','meanGMM','seGMM')
end