% Test objective function
clear; clc;
savename = mfilename;
warning('off')

% Save shocks to test Fortran code
saveshocks = 1;

% Set IES
P.psi = 2.5;

% Use specified parameters, comment out to use values in F.guess
% Phi = [...
%        ];

% Load GMM estimates
load('../GMM/gmm_data')

% Parameters
F.guess = {...  
    'gamma',	5,      0.5,      1,      1000;     % RA   
    'beta',     0.9975, 0.001,    1e-8,   0.9995;   % Discount factor
    'muy',      0.002,  0.00025,  1e-8,   1;        % Mean endowment growth
    'mud',      0.002,  0.00025,  1e-8,   1;        % Mean dividend growth
    'sigy',     0.005,  0.0025,   1e-8,   1;        % SD endowment growth shock
    'psid',     3,      1,        0,      100;      % SD dividend growth
    'pidy',     0,      0.5,      -100,   100;      % Corr. endowment and dividend growth 
    'phid',     4,      1,        1e-8,   100;      % Long-run corr. endowment and dividend growth
    'rhox',     0.98,   0.05,     1e-8,   0.9995;   % Long-run risk persistence
    'psix',     0.05,   0.025,    1e-8,   0.2};     % SD long-run risk shock
F.nparam = size(F.guess,1);

% Empirical targets
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag];

% GMM estimates
PsiD = PsiD(O.smmidx);
OmegaD = OmegaD(O.smmidx,O.smmidx);

% Parameterization and shocks
if ~exist('Phi')
	Phi = cell2mat(F.guess(:,2));
end
mtstream = RandStream('mt19937ar','seed',123456);
RandStream.setGlobalStream(mtstream);
nreps = 1000;
epsy = randn(ndatapers,nreps);
epsd = randn(ndatapers,nreps);
epsx = randn(ndatapers,nreps);
x0 = randn(1,nreps);
if saveshocks
    epsytemp = epsy(:);
    epsdtemp = epsd(:);
    epsxtemp = epsx(:);
    x0temp = x0(:);
    save('../../Fortran/LRR-noVR-noSV/test/epsy.txt','epsytemp','-ASCII','-double')
    save('../../Fortran/LRR-noVR-noSV/test/epsd.txt','epsdtemp','-ASCII','-double')
    save('../../Fortran/LRR-noVR-noSV/test/epsx.txt','epsxtemp','-ASCII','-double')
    save('../../Fortran/LRR-noVR-noSV/test/x0.txt','x0temp','-ASCII','-double')
end

% Objective function
V = variables_cs;
Vlin = variables;

% Objective function
[J,PsiM] = objfunc(...
    Phi,PsiD,OmegaD,M,O,P,F,V,Vlin,...
    ndatapers,nreps,...
    epsy,epsd,epsx,x0,1);
disp(['J =    ', num2str(J)])