function V = variables

% Variables: Names variables and assigns locations
% Inputs:
%     [None]
% Output:
%     V     :   Structure of variable locations and names

% Variables names
V.names = { 'zy' 	%1  Price-endowment ratio
            'zd'  	%2  Price-dividend ratio
            'Dy'    %3  Endowment first difference
            'Dd'    %4  Dividend first difference
            'x'     %5  Long-run risk
          };      
V.nvar = length(V.names);   

% Variables titles        
V.desc = {  'Price-Endowment Ratio'
            'Price-Dividend Ratio'
            'Endowment First Difference'
            'Dividend First Difference'
            'Long-Run Risk'
         };
                 
% Shocks
V.shocktypes = {'y','d','x'};
V.nshock = length(V.shocktypes);

% Forecast errors
V.foretypes = {'Dy','zy','Dd','zd'}; 
V.nfore = length(V.foretypes);
             
% Add expectation variables
for j = 1:V.nfore
    V.names = [V.names; ['e' V.foretypes{j}]];
end
V.nvar = length(V.names);

% Establish variable index
%  Linear model
for j = 1:V.nvar
   eval(['V.' V.names{j} ' = j;']);      
end
%  Establish shock index
for j = 1:V.nshock
    eval(['V.eps' V.shocktypes{j} ' = j;'])
end
%  Establish forecast error index
for j = 1:V.nfore
    eval(['V.fe' V.foretypes{j} ' = j;'])
end