function [zgrid, P] = rouwenhorst(muz,rho,sigeps,n)

%[zgrid, P] = rouwenhorst(muz,rho,sigeps,n)
%
% Inputs: 
%   muz 	: unconditional mean of the AR(1) process
%   rho 	: 1st order autocorrelation
%   sigeps	: standard deviation of the error term
%   n       : number of nodes in the discrete approximation
%
% Outputs: 
%   zgrid   : nodes
%   P       : transition matrix
%
% Downloaded from http://karenkopecky.net/ and modified to include muz as
% input

% Nodes
sigz = sigeps/sqrt((1-rho^2));
bound = sqrt(n-1)*sigz;
zgrid = muz + linspace(-bound,bound,n)';

% Transition Matrix
q = (rho+1)/2;
P = [q 1-q;1-q q];
for i = 2:n-1
   P = q*[P zeros(i,1);zeros(1,i+1)] + (1-q)*[zeros(i,1) P;zeros(1,i+1)] + ...
       (1-q)*[zeros(1,i+1); P zeros(i,1)] + q*[zeros(1,i+1); zeros(i,1) P];
   P(2:i,:) = P(2:i,:)/2;
end

% % Alternative calculation for Fortran
% P = zeros(n,n);
% Ptemp = zeros(n,n);
% q = (rho+1)/2;
% P(1:2,1:2) = [q 1-q;1-q q];
% for i=2:n-1
%    Ptemp(1:i,1:i) = P(1:i,1:i);
%    P(1:i,1:i) = q*Ptemp(1:i,1:i);
%    P(1:i,2:i+1) = P(1:i,2:i+1) + (1-q)*Ptemp(1:i,1:i);
%    P(2:i+1,1:i) = P(2:i+1,1:i) + (1-q)*Ptemp(1:i,1:i);
%    P(2:i+1,2:i+1) = P(2:i+1,2:i+1) + q*Ptemp(1:i,1:i);
%    P(2:i,:) = P(2:i,:)/2;
% end