function varargout = subplotpad(figh,pad)

% subplotpad(figh,pad) 
%   Adjust subplot positions
%   Inputs:
%       figh: Figure handle
%       pad: Structure with subplot padding, e.g.,
%           pad.topmarg = .11;
%           pad.leftmarg = .075;
%           pad.vspace = .065;
%           pad.hspace = .07;
%           pad.rightmarg = .02;
%           pad.botmarg = .04;
%       (units are ratio of horiztonal and vertical space)

% Get subplot handles and positions
% subh = get(figh,'Children');
subh = findall(gcf,'type','axes');
positions = cat(1,subh.Position);
nsub = numel(subh);
ncols = numel(unique(positions(:,1)));
nrows = numel(unique(positions(:,2)));

% Adjust subplot positions
for isub = 1:nsub
    [icol,irow] = ind2sub([ncols,nrows],isub);
    width = (1-pad.leftmarg-pad.hspace*(ncols-1)-pad.rightmarg)/ncols;
    height = (1-pad.topmarg-pad.vspace*(nrows-1)-pad.botmarg)/nrows;
    left = pad.leftmarg + (icol-1)*(width+pad.hspace);
    bottom = 1-pad.topmarg-irow*height-(irow-1)*pad.vspace;
    subplot(nrows,ncols,isub)
    xlims = get(gca,'xlim');
    ylims = get(gca,'ylim');
    set(gca,'Position',[left,bottom,width,height]);
    boxes(isub,:) = [left,bottom,width,height];
    set(gca,'xlim',xlims,'ylim',ylims)
end
varargout{1} = boxes;