% Baseline model estimates w.r.t. IES
% clear; close all; clc
clear pad
plotops
saving = 1;
loadflag = 1;

% Set IES
psis = 0.05:.05:3;
npsi = numel(psis);
ntaskidtot = 50;

% Options
nreps = 1000;
TSs = {'noTS','noTS','noTS'};
prefs = {'current','current','revised'};
fixgam = {'','-gam5',''};
% TSs = {'noTS'};
% prefs = {'revised'};
% fixgam = {''};
Fnmoments = 29;
nmods = numel(TSs);
paramstrs = {...
    '$\beta$','$\rho_a$','$\sigma_a$',...
    '$\mu_y$','$\mu_d$','$\sigma_y$','$\psi_d$','$\pi_{dy}$',...
    '$J$','pval'};
Fnparam = numel(paramstrs)-2;
coefstrs = {...
    '$\kappa_{y,0}$','$\kappa_{y,1}$',...
    '$\eta_{y,0}$','$\eta_{y,1}$',...
    '$\kappa_{d,0}$','$\kappa_{d,1}$',...
    '$\eta_{d,0}$','$\eta_{d,1}$'};
Fncoef = numel(coefstrs);

% Load GMM estimates
load('GMM/gmm_data')

%   Empirical targets
smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd];
Fnestmom = numel(smmidx);

% GMM estimates
PsiDSE = sqrt(diag(OmegaD));
moms = M.desc;

if ~loadflag
    ntotmissing = 0;
    meanPhis = NaN(Fnparam,nmods,npsi);
    meanJs = NaN(nmods,npsi);
    meanPsis = NaN(Fnmoments,nmods,npsi);
    meancoefs = NaN(Fncoef,nmods,npsi);
    for ipsi = 1:npsi
        Ppsi = psis(ipsi);
        for imod = 1:nmods
            % Load parameters, obj. func. values, and moments
            fnprefix = ['../Fortran/Baseline-fixgamma/rslt/',...
                prefs{imod},'-',TSs{imod},'-psi',num2str(Ppsi,'%.2f'),fixgam{imod}];
            if exist([fnprefix,'-Phis.txt'],'file')
                file = dir([fnprefix '-Phis.txt']);
                disp([file.name, ' ', file.date])
                Phis = reshape(textread([fnprefix,'-Phis.txt'],'%f'),[ntaskidtot,Fnparam]);
                Js = textread([fnprefix '-Js.txt'],'%f');
                Psis = reshape(textread([fnprefix,'-Psis.txt'],'%f'),[ntaskidtot,Fnmoments]);
                coefs = reshape(textread([fnprefix,'-coefs.txt'],'%f'),[ntaskidtot,Fncoef]);

                % Truncate zeros
                imissing = Phis(:,strcmp(paramstrs,'$\beta$'))==0;
                if strcmp(prefs{imod},'revised')
                    imissing = imissing | ...
                        Phis(:,strcmp(paramstrs,'$\sigma_a$')) < 0.03;
                end
                Phis(imissing,:) = [];
                Js(imissing) = [];
                Psis(imissing,:) = [];
                coefs(imissing,:) = [];
                ntaskid = numel(Js);
                disp([' # finished tasks = ',num2str(ntaskid)])
                if any(imissing)
                    disp(' missing tasks')
                    disp(['    ',num2str(find(imissing)','%4.0f')])
                end
                ntotmissing = ntotmissing + sum(imissing);

                % Parameters
                meanPhis(:,imod,ipsi) = mean(Phis);
                % Obj. func. values
                Js = nreps*Js/(1+nreps);
                meanJs(imod,ipsi) = mean(Js);
                % Moments
                meanPsis(:,imod,ipsi) = mean(Psis);
                % Solution coefficients
                meancoefs(:,imod,ipsi) = mean(coefs);
            else
                meanPhis(:,imod,ipsi) = NaN;
                meanJs(imod,ipsi) = NaN;
                meanPsis(:,imod,ipsi) = NaN;
                meancoefs(:,imod,ipsi) = NaN;
            end
        end
    end
    disp(['total # missing tasks = ',num2str(ntotmissing)])
    save('Results/baseline-IES.mat','meanPhis','meanJs','meanPsis','meancoefs')
else
    load('Results/baseline-IES.mat');
end

%% Figure

% Plot options
figpos = [1 4.5 6.5 4];
pad.topmarg = .11;
pad.leftmarg = .05;
pad.vspace = .095;
pad.hspace = .055;
pad.rightmarg = .06;
pad.botmarg = .08;
legfontsize = 10;
legadj = .075;
legheight = .01;
linetypes = {'kx','rd','bo'};
xlims = [0.45,2.5];
xticks = .5:.25:3;
legstrs = {...
    'Current ($\gamma = 1.58$)',...
    'Current ($\gamma = 5$)',...
    'Revised ($\gamma = 75.11$)'};
legstrs2 = {...
    'Right Axis'};
markersizes = [6,4,4];
savename = 'Results/baseline_fixgamma_custom';

clear plotme
j = 0;
j=j+1; plotme(:,:,j) = meanJs;
j=j+1; plotme(:,:,j) = squeeze(meanPhis(strcmp(paramstrs,'$\sigma_a$'),:,:));
j=j+1; plotme(:,:,j) = squeeze(meanPhis(strcmp(paramstrs,'$\psi_d$'),:,:));
j=j+1; plotme(:,:,j) = squeeze(meanPhis(strcmp(paramstrs,'$\pi_{dy}$'),:,:));
titlestrs = {...
    'Model Fit ($J$)',...
    'Preference Shock Volatility ($\sigma_a$)',...
    'Dividend Growth Volatility ($\psi_d$)',...
    'Endowment and Dividend Comovement ($\pi_{dy}$)'};
plotdim = [2,2];

figure
set(gcf,'Position',figpos,...
    'PaperPosition',[0 0 figpos(3:4)],'PaperSize',figpos(3:4));
p1 = NaN(nmods,numel(plotdim));
for isubplot = 1:size(plotme,3)
    subplot(plotdim(1),plotdim(2),isubplot); grid on; box on; hold on;
    [icol,irow] = ind2sub(plotdim([2,1]),isubplot);
    for imod = 1:nmods
        if imod == 2 && isubplot == 2
            ipsis = find(psis>= 1.2 | psis< 1);
        elseif imod == 3 && isubplot == 2
            ipsis = find(psis>= 1.1 | psis< 1);
        else
            ipsis = find(psis>0);
        end
        if imod == nmods && isubplot == 2
            yyaxis right
            p1(imod,isubplot) = plot(psis(ipsis),plotme(imod,ipsis,isubplot),linetypes{imod},...
            'markersize',markersizes(imod));
        else
            p1(imod,isubplot) = plot(psis(ipsis),plotme(imod,ipsis,isubplot),linetypes{imod},...
            'markersize',markersizes(imod));
        end
    end
    if isubplot == 2
        ax = gca;
        ax.YAxis(1).Color = 'k';
        ax.YAxis(2).Color = 'b';
    end
    title(titlestrs{isubplot})
    if irow == plotdim(1)
        xlabel('IES ($\psi$)')
    end
    set(gca,'xlim',xlims,'xtick',xticks);
end
subplotpad(gcf,pad);
legend(p1(:,1),legstrs,...
    'Orientation','horizontal',...
    'Location','NorthEast',...
    'Position',[.25,1-pad.topmarg+legadj,0.5,legheight],...
    'FontSize',legfontsize,'interpreter','latex');
legend(p1(nmods,2),legstrs2,...
    'Orientation','horizontal',...
    'Location','NorthEast',...
    'FontSize',8,'interpreter','latex');

% Save figure
if saving
    saveas(gcf,[savename,'.pdf'])
    saveas(gcf,[savename,'.fig'])
end