% State-space indifference curves
% clear; close all; clc
plotops
saving = 'on';

% Subplot 1
isubplot = 1;
prefs{isubplot} = 'C';
gammas{isubplot} = [0.1,1.5,3];
psis{isubplot} = [0.95,0.95,0.95];
titles{isubplot} = 'Current ($\psi=0.95$)';
legstrs{isubplot} = {'$\gamma=0.1$','$\gamma=1.5$','$\gamma=3$'};
% Subplot 2
isubplot = 2;
prefs{isubplot} = 'C';
gammas{isubplot} = [0.1,1.5,3];
psis{isubplot} = [1.05,1.05,1.05];
titles{isubplot} = 'Current ($\psi=1.05$)';
legstrs{isubplot} = {'$\gamma=0.1$','$\gamma=1.5$','$\gamma=3$'};
% Subplot 3
isubplot = 3;
prefs{isubplot} = 'R';
gammas{isubplot} = [3,3,3];
psis{isubplot} = [0.5,1.05,1.5];
str0 = strcat('$\gamma=',num2str(gammas{isubplot}(1)),'$');
str1 = strcat('$\psi=',num2str(psis{isubplot}(1)),'$');
str2 = strcat('$\psi=',num2str(psis{isubplot}(2)),'$');
str3 = strcat('$\psi=',num2str(psis{isubplot}(3)),'$');
legstrs{isubplot} = {str1,str2,str3};
titles{isubplot} = strcat('Revised (',str0,')');
% Subplot 4
isubplot = 4;
prefs{isubplot} = 'R';
gammas{isubplot} = [3,9,27];
psis{isubplot} = [0.95,0.95,0.95];
str0 = strcat('$\psi=',num2str(psis{isubplot}(1)),'$');
str1 = strcat('$\gamma=',num2str(gammas{isubplot}(1)),'$');
str2 = strcat('$\gamma=',num2str(gammas{isubplot}(2)),'$');
str3 = strcat('$\gamma=',num2str(gammas{isubplot}(3)),'$');
legstrs{isubplot} = {str1,str2,str3};
titles{isubplot} = strcat('Revised (',str0,')');

nsubplot = isubplot;
ncurves = numel(gammas{1});

% Valuation risk state 1
a1 = [linspace(.8,.98,1000),linspace(.98,1.02,10000),linspace(1.02,1.2,1000)];
na1 = numel(a1);

% Parameters universal
P.beta = 0.9975;
P.Delta = 1.0015;  

a2 = NaN(na1,ncurves,nsubplot);
for isubplot = 1:nsubplot
    % Preferences
    P.pref = prefs{isubplot};
    for icurve = 1:ncurves
        % Parameters
        %   Curve specific
        P.psi   = psis{isubplot}(icurve);
        P.gamma	= gammas{isubplot}(icurve);
        %   Implied
        P.rho   = 1-1/P.psi;
        P.alpha = 1-P.gamma;
        P.xbar = P.Delta^P.rho*(1-P.beta)/(1-P.beta*P.Delta^P.rho);
        P.Ubar  = ((1-P.beta)+P.beta*P.xbar)^(1/P.rho);
        % Valuation risk state 2
        for ia1 = 1:length(a1)
            if strcmp(P.pref,'C')
                temp = 2*P.Ubar^P.alpha - ((1-P.beta) + a1(ia1)*P.beta*P.xbar)^(P.alpha/P.rho);
                if temp > 0
                    a2(ia1,icurve,isubplot) = (temp^(P.rho/P.alpha)-(1-P.beta))/(P.beta*P.xbar);
                end
            elseif strcmp(P.pref,'R')
                temp = 2*P.Ubar^P.alpha - ((1 - a1(ia1)*P.beta) + a1(ia1)*P.beta*P.xbar)^(P.alpha/P.rho);
                if temp > 0 
                    a2(ia1,icurve,isubplot) = (temp^(P.rho/P.alpha)-1)/(P.beta*(P.xbar-1));
                end
            end
        end
    end
end

%% Figure

% Plot options
ifig=0;
% 	Manuscript version
ifig=ifig+1;
figpos{ifig} = [1,1.5,6.5,4.5];
pad(ifig).topmarg = .05;
pad(ifig).leftmarg = .075;
pad(ifig).vspace = .10;
pad(ifig).hspace = .06;
pad(ifig).rightmarg = .02;
pad(ifig).botmarg = .09;
fontsize(ifig) = 10;
legfontsize(ifig) = 10;
legadj(ifig) = .1;
figtypes{ifig} = '';
% % 	Presentation version
% ifig=ifig+1;
% figpos{ifig} = [1,1.5,4.5,2.6];
% pad(ifig).topmarg = .05;
% pad(ifig).leftmarg = .075;
% pad(ifig).vspace = .10;
% pad(ifig).hspace = .06;
% pad(ifig).rightmarg = .02;
% pad(ifig).botmarg = .10;
% fontsize(ifig) = 7;
% legfontsize(ifig) = 7;
% legadj(ifig) = .1;
% figtypes{ifig} = '_pres';

% Common options
linetypes = {'k-','b--','r-.'};

% Plot
for ifig = 1:numel(figtypes)
    figure
    set(gcf,'Position',figpos{ifig},...
        'PaperPosition',[0 0 figpos{ifig}(3:4)],...
        'PaperSize',figpos{ifig}(3:4));
    h = zeros(ncurves,nsubplot);
    for isubplot = 1:nsubplot
        subplot(2,2,isubplot); hold on; grid on; box on;
        plot(a1,a1,'--k','LineWidth',2)
        for icurve = 1:ncurves
            h(icurve,isubplot) = plot(a1,a2(:,icurve,isubplot),linetypes{icurve});
        end
        axis([min(a1),max(a1),min(a1),max(a1)])
        if any(isubplot==[3,4])
            xlabel('Valuation Risk State 1 ($a_1$)')
        end
        if any(isubplot==[1,3])
            ylabel('Valuation Risk State 2 ($a_2$)')
        end
        title(titles{isubplot})
    end
    %   Adjust subplot padding
    subplotpad(gcf,pad(ifig))
    set(findall(gcf,'-property','FontSize'),'FontSize',fontsize(ifig))
    %   Add legends
    for isubplot = 1:nsubplot
        legend(h(:,isubplot),legstrs{isubplot},'FontSize',legfontsize(ifig))
    end
    set(findall(gcf,'-property','interpreter'),'interpreter','latex')

    % Save
    if strcmp(saving,'on')
        saveas(gcf,['Results/indifference_curves',figtypes{ifig},'.pdf'])
    end
end