% LRR models parameters and moments
% clear; close all; clc
plotops

% Save tables
loadflag = 1;
save_tex = 1;
scatterplots = 0;
disptables = 0;

% Set IES
psis = [1.5,2.0,2.5];

% Options
ntaskids = 500;
nreps = 1000;
prefs = {'','revised-','','revised-','','revised-'};
TSs = {'norf','norf','noTS','noTS','TS','TS'};
mods = {'LRR-noVR','LRR-VR','LRR-noVR','LRR-VR','LRR-noVR','LRR-VR'};
nicknames = {'norf_noVR','norf_VR','noTS_noVR','noTS_VR','TS_noVR','TS_VR'};
Fnparams = [10,12,10,12,10,12];
Fnestmoms = [18,18,20,20,22,22];
Fnmoments = 29;
nmods = numel(mods);
params = {...
    '$\gamma$','$\beta$','$\rho_a$','$\sigma_a$',...
    '$\mu_y$','$\mu_d$','$\sigma_y$','$\psi_d$','$\pi_{dy}$',...
    '$\phi_d$','$\rho_x$','$\psi_x$','$J$','pval'};
Fnparam = max(Fnparams);

% Load GMM estimates
load('GMM/gmm_data')

%   Empirical targets
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag];
imomord = [1,2,5,3,4,6:numel(O.smmidx)];
Fnestmom = numel(O.smmidx);

% GMM estimates
PsiDSE = sqrt(diag(OmegaD));
moms = M.desc;

for ipsi = 1:numel(psis)
    Ppsi = psis(ipsi);
    matname = ['results/LRR-psi',num2str(Ppsi,'%.1f'),'.mat'];
    if ~loadflag
        clear dfs tab
        dfs = zeros(1,nmods);
        tab = cell(2,nmods);
        for imod = 1:nmods
            % Load parameters, obj. func. values, and moments
            fnprefix = ['../Fortran/',mods{imod},'/rslt/',...
                prefs{imod},TSs{imod},'-psi',num2str(Ppsi,'%.1f')];
            if exist([fnprefix,'-Phis.txt'],'file')
                file = dir([fnprefix,'-Phis.txt']);
                disp([fnprefix,' ', file.date])
                Phis = reshape(textread([fnprefix,'-Phis.txt'],'%f'),[ntaskids,Fnparams(imod)]);
                Js = textread([fnprefix,'-Js.txt'],'%f');
                Psis = reshape(textread([fnprefix,'-Psis.txt'],'%f'),[ntaskids,Fnmoments]);

                % Truncate zeros
                imissing = Phis(:,1)==0;
                Phis(imissing,:) = [];
                Js(imissing) = [];
                Psis(imissing,:) = [];
                ntaskid = numel(Js);
                disp([' # finished tasks = ',num2str(ntaskid)])
                if any(imissing)
                    disp(' missing tasks')
                    disp(['    ',num2str(find(imissing)','%4.0f')])
                end
                ntotmissing = ntotmissing + sum(imissing);

                % Means, SDs, and percentiles
                if Fnparams(imod) == 10
                    Phis = [Phis(:,1:2),NaN(ntaskid,2),Phis(:,3:end)];
                end
                %   Parameters
                meanPhis = mean(Phis);
                SDPhis = std(Phis);
                perPhis = quantile(Phis,[.05,.95]);
                %   Obj. func. values
                Js = nreps*Js/(1+nreps);
                meanJs = mean(Js);
                SDJs = std(Js);
                perJs = quantile(Js,[.05,.95]);
                dfs(imod) = Fnestmoms(imod) - Fnparams(imod);
                Jpvals = chi2cdf(Js,dfs(imod),'upper');
                meanJpvals = mean(Jpvals);
                SDJpvals = std(Jpvals);
                perJpvals = quantile(Jpvals,[.05,.95]);
            %     %   T-stats
            %     if Fnestmoms(imod) == 18
            %         Psis = [Psis(:,1:8),NaN(ntaskid,1),Psis(:,9:12),NaN(ntaskid,1),Psis(:,13:end)];
            %     end
                meanPsis = mean(Psis);
                tstats = (Psis - repmat(PsiD',[ntaskid,1]))./repmat(PsiDSE',[ntaskid,1]);
                meantstat = (meanPsis - PsiD')./PsiDSE';
                pvals = 2*tcdf(-abs(tstats),ntaskid-1);
                meanpval = mean(pvals);
                rejper = 100*sum(pvals<.05,1)/ntaskid;

                % Table of parameter estimates
                tab{1,imod} = table([meanPhis,meanJs,meanJpvals]',[SDPhis,SDJs,SDJpvals]',...
                        [perPhis(1,:),perJs(1),perJpvals(1)]',[perPhis(2,:),perJs(2),perJpvals(2)]');
                tab{1,imod}.Properties.VariableNames = ...
                    strcat(TSs{imod},'_',mods{imod},{'_mean','_SD','p5','p95'});
                tab{1,imod}.Properties.RowNames = params;

                % Table of moments
                tab{2,imod} = table(PsiD,PsiDSE,meanPsis',meantstat',meanpval',rejper');
                tab{2,imod}.Properties.VariableNames = ...
                    ['data','data_SE',strcat(TSs{imod},'_',mods{imod},{'_mean','_tstat','_pval','_RejPer'})];
                tab{2,imod}.Properties.RowNames = moms;
            else
                tab{1,imod} = array2table(NaN(Fnparam+2,4));
                tab{2,imod} = array2table(NaN(Fnmoments,6));
            end

            % Check Js for outliers
            if scatterplots
                figure; set(gcf,'position',[.25+(imod-1)*3,1,3,4],'name',[nicknames{imod},' psi = ',num2str(Ppsi)]);
                ncol = 3;
                nrow = ceil(Fnparam/ncol);
                for iparam = 1:Fnparam
                    subplot(nrow,ncol,iparam); grid on; box on; hold on;
                    plot(Phis(:,iparam),Js,'r.');
                    title(params{iparam})
                end
            end
        end
        save(matname,'tab','dfs')
    else
        load(matname)
    end

    if disptables
        disp('-------------------------------------------------------------------------------------')
        disp('Omits sd(rf) and ac(rf)')
        disp('-------------------------------------------------------------------------------------')

        disp('    Parameter estimates')
        format shortg; disp([tab{1,1}(:,1:2),tab{1,2}(:,1:2)])

        disp('    Moments estimates')
        format bank; disp([tab{2,1}(:,[1,2,3,4]),tab{2,2}(:,[3,4])])

        disp('-------------------------------------------------------------------------------------')
        disp('All moments')
        disp('-------------------------------------------------------------------------------------')

        disp('    Parameter estimates')
        format shortg; disp([tab{1,3}(:,1:2),tab{1,4}(:,1:2)])

        disp('    Moments estimates')
        format bank; disp([tab{2,3}(:,[1,2,3,4]),tab{2,4}(:,[3,4])])
    end

    %% Create tables for LaTeX

    % -------------------------------------------------------------------------------------
    %    Parameter estimates
    % -------------------------------------------------------------------------------------
    savename = ['LRR-psi' num2str(Ppsi,'%.1f') '-parameters'];
    ncols = nmods + 1;

    % Header
    header = cell(6,ncols);
    header{1,1} = ['\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}*{',num2str(1+nmods),'}c}'];
    header{2,1} = '\toprule';
    header{3,1} = ' & \multicolumn{2}{c}{Omits $SD[r_f]$, $AC[r_f]$,} & \multicolumn{2}{c}{Omits} & \multicolumn{2}{c}{All Moments}  \\';
    header{4,1} = ' & \multicolumn{2}{c}{$E[r_{f,5}]$, \& $E[r_{f,20}]$} & \multicolumn{2}{c}{$E[r_{f,5}]$ \& $E[r_{f,20}]$} & \multicolumn{2}{c}{}  \\[2pt]';
    header{5,1} = ' \cline{2-3} \cline{4-5} \cline{6-7} \rule{0pt}{\normalbaselineskip}\linebreak';
    header{6,1} = 'Parameter & No VR & Revised & No VR & Revised & No VR & Revised \\';
    header{7,1} = '\midrule';

    % Entries
    nformat = {...
        '%.2f','%.4f','%.4f','%.4f','%.4f','%.4f','%.4f',...
        '%.2f','%.3f','%.2f','%.4f','%.4f','%.2f','%.3f'};
    entries = [];
    for ipar = 1:numel(params)
        entry = [];
        % Column 1
        if ipar <= Fnparam || ipar == numel(params)
            midrule = 0;
        else
            midrule = 1;
            entry{1,1} = '\midrule';
        end
        entry{midrule+1,1} = params{ipar};
        % Columns 2+
        for imod = 1:nmods
            if ~isnan(table2array(tab{1,imod}(ipar,1)))
                % Row 1: mean
                entry{midrule+1,1+imod} = ['$',...
                    num2str(table2array(tab{1,imod}(ipar,1)),nformat{ipar}),'$'];
                % Row 2: percentiles
                entry{midrule+2,1+imod} = ['\scs$(',...
                    num2str(table2array(tab{1,imod}(ipar,3)),nformat{ipar}),',',...
                    num2str(table2array(tab{1,imod}(ipar,4)),nformat{ipar}),')$'];
            else
                entry{midrule+1,1+imod} = '{$-$}';
                entry{midrule+2,1+imod} = ['',''];
            end
        end
        % Concatenate entries
        entries = [entries;entry];
    end

    % Add line breaks, cell breaks, and spacing
    entries(1:end-5,end) = strcat(entries(1:end-5,end),' \\ ');
    entries(1:end-5,1:end-1) = strcat(entries(1:end-5,1:end-1),' & ');
    entries(1:2:end-5,end) = strcat(entries(1:2:end-5,end),' [-5pt] ');

    entries(end-3:end,end) = strcat(entries(end-3:end,end),' \\ ');
    entries(end-3:end,1:end-1) = strcat(entries(end-3:end,1:end-1),' & ');
    entries(end-3:2:end,end) = strcat(entries(end-3:2:end,end),' [-5pt] ');

    % Add df
    entries(end+1,:) = cellstr(['df',strcat('&',string(dfs))]);
    entries(end,end) = strcat(entries(end,end),' \\ ');

    % Footer
    footer = cell(2,ncols);
    footer{1,1} = '\bottomrule';
    footer{2,1} = '\end{tabular*}';

    % Concatenate 
    tablatex = [header;entries;footer];

    % Save as *.tex
    if save_tex
        fid = fopen(['Results/',savename,'.tex'],'w');
        [nrows,ncols] = size(tablatex);
        tabformat = [repmat('%s ',1,ncols),' \n'];
        for row = 1:nrows
            fprintf(fid,tabformat,tablatex{row,:});
        end
        fclose(fid);
    end

    % -------------------------------------------------------------------------------------
    %    Moments
    % -------------------------------------------------------------------------------------
    savename = ['LRR-psi' num2str(Ppsi,'%.1f') '-moments'];
    momstrs = {'E[\Delta c]','E[\Delta d]','E[r_d]','E[r_f]','E[z_d]','E[r_{f,5}]','E[r_{f,20}]',...
        'SD[\Delta c]','SD[\Delta d]','SD[r_d]','SD[r_f]','SD[z_d]',...
        'AC[\Delta c]','AC[\Delta d]',...
        'AC[r_d]','AC[r_f]','AC[z_d]',...
        'Corr[\Delta c,\Delta d]','Corr[\Delta c,r_d]','Corr[\Delta d,r_d]',...
        'Corr[ep,z_{d,-1}]','Corr[\Delta c,z_{d,-1}]'};
    ncols = 2 + nmods;

    % Header
    header = cell(6,ncols);
    header{1,1} = ['\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}l*{',num2str(1+nmods),'}{c}}'];
    header{2,1} = '\toprule';
    header{3,1} = ' & & \multicolumn{2}{c}{Omits $SD[r_f]$, $AC[r_f]$,} & \multicolumn{2}{c}{Omits} & \multicolumn{2}{c}{All Moments}  \\';
    header{4,1} = ' & & \multicolumn{2}{c}{$E[r_{f,5}]$, \& $E[r_{f,20}]$} & \multicolumn{2}{c}{$E[r_{f,5}]$ \& $E[r_{f,20}]$} & \multicolumn{2}{c}{}  \\[2pt]';
    header{5,1} = '             \cline{3-4}                       \cline{5-6} \cline{7-8} \rule{0pt}{\normalbaselineskip}\linebreak';
    header{6,1} = '{Moment} & {Data} & {No VR} & {Revised} & {No VR} & {Revised} & {No VR} & {Revised} \\';
    header{7,1} = '\midrule';

    % Entries
    entries = [];
    entry = [];
    for imom = 1:Fnestmom
        ismmidx = O.smmidx(imomord(imom));
        % Column 1
        entry{1,1} = ['$',momstrs{imomord(imom)},'$'];
        % Column 2: Data
        entry{1,2} = ['\num{', ... 
            num2str(table2array(tab{2,1}(ismmidx,1)),'%.2f'),'}'];
        % Columns 4+: Model means
        for imod = 1:imod
            if ~isnan(table2array(tab{2,imod}(ismmidx,3)))
                entry{1,2+imod} = ['\num{', ... 
                    num2str(table2array(tab{2,imod}(ismmidx,3)),'%.2f'),'}'];
                entry{2,2+imod} = ['\Tiny(\num{', ... 
                    num2str(table2array(tab{2,imod}(ismmidx,4)),'%.2f'),'})'];
            else
                entry{1,2+imod} = '{$-$}';
                entry{2,2+imod} = '';
            end
        end
        % Concatenate entries
        entries = [entries;entry];
    end

    % Add line breaks, cell breaks, and spacing
    entries(:,end) = strcat(entries(:,end),' \\ ');
    entries(:,1:end-1) = strcat(entries(:,1:end-1),' & ');
    entries(1:2:end,end) = strcat(entries(1:2:end,end),' [-5pt] ');

    % Footer
    footer = cell(2,ncols);
    footer{1,1} = '\bottomrule';
    footer{2,1} = '\end{tabular*}';

    % Concatenate 
    tablatex = [header;entries;footer];

    % Save as *.tex
    if save_tex
        fid = fopen(['Results/',savename,'.tex'],'w');
        [nrows,ncols] = size(tablatex);
        tabformat = [repmat('%s ',1,ncols),' \n'];
        for row = 1:nrows
            fprintf(fid,tabformat,tablatex{row,:});
        end
        fclose(fid);
    end
end