% Extended models parameters and moments
% clear; close all; clc
plotops

% Save tables
loadflag = 1;
save_tex = 1;
scatterplots = 0;
disptables = 0;

% Set IES
psis = [1.5,2.0,2.5];

% Options
ntaskids = 500;
nreps = 1000;
prefs = {'','revised-','revised-','','revised-','revised-'};
TSs = {'noTS','noTS','noTS','TS','TS','TS'};
mods = {'Extended-noVR-SV','Extended-Demand','Extended-Demand-SV',...
    'Extended-noVR-SV','Extended-Demand','Extended-Demand-SV'};
nicknames = {...
    'noTS_noVR_SV','noTS_Demand','noTS_Demand_SV',...
    'TS_noVR_SV','TS_Demand','TS_Demand_SV'};
Fnparams = [12,12,14,12,12,14];
Fnestmoms = [20,20,20,22,22,22];
Fnmoments = 29;
nmods = numel(mods);
params = {...
    '$\gamma$','$\beta$','$\rho_a$','$\sigma_a$',...
    '$\mu_y$','$\mu_d$','$\sigma_y$','$\psi_d$','$\pi_{dy}$',...
    '$\phi_d$','$\rho_x$','$\psi_x$','$\pi_{ya}$','$\pi_{da}$',...
    '$\rho_{\sigma_y}$','$\nu_y$','$J$','pval'};
paramord = 1:20;
Fnparam = max(Fnparams);

% Load GMM estimates
load('GMM/gmm_data')

%   Empirical targets
O.smmidx = [...
    M.mu_gc,M.mu_gd,M.mu_rd,M.mu_rf,M.mu_zd,M.mu_rf5,M.mu_rf20,...
    M.sd_gc,M.sd_gd,M.sd_rd,M.sd_rf,M.sd_zd,...
    M.ac_gc,M.ac_gd,M.ac_rd,M.ac_rf,M.ac_zd,...
    M.corr_gcgd,M.corr_gcrd,M.corr_gdrd,...    
    M.corr_ep1zdlag,M.corr_gc1zdlag];
imomord = [1,2,5,3,4,6:numel(O.smmidx)];
Fnestmom = numel(O.smmidx);

% GMM estimates
PsiDSE = sqrt(diag(OmegaD));
moms = M.desc;

ntotmissing = 0;
for ipsi = 1:numel(psis)
    Ppsi = psis(ipsi);
    matname = ['results/extended-psi',num2str(Ppsi,'%.1f'),'.mat'];
    if ~loadflag
        clear dfs tab
        dfs = zeros(1,nmods);
        tab = cell(2,nmods);
        for imod = 1:nmods
            % Load parameters, obj. func. values, and moments
            fnprefix = ['../Fortran/',mods{imod},'/rslt/',...
                prefs{imod},TSs{imod},'-psi',num2str(Ppsi,'%.1f')];
            if exist([fnprefix,'-Phis.txt'],'file')
                file = dir([fnprefix,'-Phis.txt']);
                disp([fnprefix,' ', file.date])
                Phis = reshape(textread([fnprefix '-Phis.txt'],'%f'),[ntaskids,Fnparams(imod)]);
                Js = textread([fnprefix '-Js.txt'],'%f');
                Psis = reshape(textread([fnprefix '-Psis.txt'],'%f'),[ntaskids,Fnmoments]);

                % Truncate zeros
                imissing = Phis(:,1)==0;
                Phis(imissing,:) = [];
                Js(imissing) = [];
                Psis(imissing,:) = [];
                ntaskid = numel(Js);
                disp([' # finished tasks = ',num2str(ntaskid)])
                if any(imissing)
                    disp(' missing tasks')
                    disp(['    ',num2str(find(imissing)','%4.0f')])
                end
                ntotmissing = ntotmissing + sum(imissing);

                % Means, SDs, and percentiles
                if strcmp(mods{imod},'Extended-noVR-SV')
                    Phis = [Phis(:,1:2),NaN(ntaskid,2),Phis(:,3:10),NaN(ntaskid,2),Phis(:,11:12)];
                elseif strcmp(mods{imod},'Extended-Demand')
                    Phis = [Phis(:,1:7),NaN(ntaskid,2),Phis(:,8:12),NaN(ntaskid,2)];
                elseif strcmp(mods{imod},'Extended-Demand-SV')
                    Phis = [Phis(:,1:7),NaN(ntaskid,2),Phis(:,8:14)];
                end
                %   Parameters
                meanPhis = mean(Phis);
                SDPhis = std(Phis);
                perPhis = quantile(Phis,[.05,.95]);
                %   Obj. func. values
                Js = nreps*Js/(1+nreps);
                meanJs = mean(Js);
                SDJs = std(Js);
                perJs = quantile(Js,[.05,.95]);
                dfs(imod) = Fnestmoms(imod) - Fnparams(imod);
                Jpvals = chi2cdf(Js,dfs(imod),'upper');
                meanJpvals = mean(Jpvals);
                SDJpvals = std(Jpvals);
                perJpvals = quantile(Jpvals,[.05,.95]);
                %   T-stats
            %     if Fnestmoms(imod) == 18
            %         Psis = [Psis(:,1:8),NaN(ntaskids(imod),1),Psis(:,9:12),NaN(ntaskids(imod),1),Psis(:,13:end)];
            %     end
                meanPsis = mean(Psis);
                tstats = (Psis - repmat(PsiD',[ntaskid,1]))./repmat(PsiDSE',[ntaskid,1]);
                meantstat = (meanPsis - PsiD')./PsiDSE';
                pvals = 2*tcdf(-abs(tstats),ntaskid-1);
                meanpval = mean(pvals);
                rejper = 100*sum(pvals<.05,1)/ntaskid;
                Jcont = ((meanPsis - PsiD')./PsiDSE').^2;

                % Table of parameter estimates
                tab{1,imod} = table([meanPhis,meanJs,meanJpvals]',[SDPhis,SDJs,SDJpvals]',...
                        [perPhis(1,:),perJs(1),perJpvals(1)]',[perPhis(2,:),perJs(2),perJpvals(2)]');
                tab{1,imod}.Properties.VariableNames = ...
                    strcat(prefs{imod},{'_mean','_SD','p5','p95'});
                tab{1,imod}.Properties.RowNames = params;

                % Table of moments
                tab{2,imod} = table(PsiD,PsiDSE,meanPsis',meantstat',meanpval',rejper',Jcont');
                tab{2,imod}.Properties.VariableNames = ...
                    ['data','data_SE',strcat(prefs{imod},{'_mean','_tstat','_pval','_RejPer','_Jcont'})];
                tab{2,imod}.Properties.RowNames = moms;
            else
                tab{1,imod} = array2table(NaN(Fnparam+2,4));
                tab{2,imod} = array2table(NaN(Fnmoments,6));
            end

            % Check Js for outliers
            if scatterplots
                figure; set(gcf,'position',[.25+(imod-1)*3,1,3,4],'name',[nicknames{imod},' psi = ',num2str(Ppsi)]);
                ncol = 3;
                nrow = ceil(Fnparam/ncol);
                for iparam = 1:Fnparam
                    subplot(nrow,ncol,iparam); grid on; box on; hold on;
                    plot(Phis(:,iparam),Js,'r.');
                    title(params{iparam})
                end
            end
        end
        save(matname,'tab','dfs')
    else
        load(matname)
    end

    % disp('-------------------------------------------------------------------------------------')
    % disp('All Moments')
    % disp('-------------------------------------------------------------------------------------')
    % 
    % disp('-------------------------------------------------------------------------------------')
    % disp('    Parameter estimates')
    % disp('-------------------------------------------------------------------------------------')
    % format shortg; disp([tab{1,1}(:,1:2),tab{1,2}(:,1:2)])
    % 
    % disp('-------------------------------------------------------------------------------------')
    % disp('    Moments estimates')
    % disp('-------------------------------------------------------------------------------------')
    % format bank; disp([tab{2,1}(:,[1,2,3,4]),tab{2,2}(:,[3,4])])
    % 
    % disp('-------------------------------------------------------------------------------------')
    % disp('Excluding risk-free rate moments')
    % disp('-------------------------------------------------------------------------------------')
    % 
    % disp('-------------------------------------------------------------------------------------')
    % disp('    Parameter estimates')
    % disp('-------------------------------------------------------------------------------------')
    % format shortg; disp([tab{1,3}(:,1:2),tab{1,4}(:,1:2)])
    % 
    % disp('-------------------------------------------------------------------------------------')
    % disp('    Moments estimates')
    % disp('-------------------------------------------------------------------------------------')
    % format bank; disp([tab{2,3}(:,[1,2,3,4]),tab{2,4}(:,[3,4])])


    %% Create tables for LaTeX

    % -------------------------------------------------------------------------------------
    %    Parameter estimates
    % -------------------------------------------------------------------------------------
    savename = ['extended-psi' num2str(Ppsi,'%.1f') '-parameters'];
    ncols = 1 + nmods;

    % Header
    header = cell(6,ncols);
    header{1,1} = '\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}*{7}{c}}';
    header{2,1} = '\toprule';
    header{3,1} = ' & \multicolumn{3}{c}{Omits $E[r_{f,5}]$ \& $E[r_{f,20}]$} & \multicolumn{3}{c}{All Moments} \\[2pt]';
    header{4,1} = ' \cline{2-4} \cline{5-7} \rule{0pt}{\normalbaselineskip}\linebreak';
    header{5,1} = 'Ptr & No VR$+$SV & Demand & Demand$+$SV & No VR$+$SV & Demand & Demand$+$SV \\';
    header{6,1} = '\midrule';

    % Entries
    nformat = {...
        '%.2f','%.4f','%.4f','%.4f','%.4f','%.4f','%.4f',...
        '%.2f','%.3f','%.2f','%.4f','%.4f','%.3f','%.3f',...
        '%.4f','%.1e','%.2f','%.3f'};
    entries = [];
    for ipar = 1:numel(params)
        entry = cell(1,ncols);
        % Column 1
        if ipar <= numel(params)-2 || ipar == numel(params)
            iparord = paramord(ipar);
            midrule = 0;
        else
            iparord = ipar;
            midrule = 1;
            entry{1,1} = '\midrule';
        end
        entry{midrule+1,1} = params{iparord};
        % Columns 2+
        for imod = 1:nmods
            if ~isnan(table2array(tab{1,imod}(iparord,1)))
                % Row 1: mean
                entry{midrule+1,1+imod} = ['\num{',...
                    num2str(table2array(tab{1,imod}(iparord,1)),nformat{iparord}),'}'];
                % Row 2: percentiles
                entry{midrule+2,1+imod} = ['\Tiny(\num{',...
                    num2str(table2array(tab{1,imod}(iparord,3)),nformat{iparord}),'}, \num{',...
                    num2str(table2array(tab{1,imod}(iparord,4)),nformat{iparord}),'})'];
            else
                entry{1,1+imod} = '{$-$}';
                entry{2,1+imod} = ['',''];
            end
        end
        % Concatenate entries
        entries = [entries;entry];
    end

    % Add line breaks, cell breaks, and spacing
    entries(1:end-5,end) = strcat(entries(1:end-5,end),' \\ ');
    entries(1:end-5,1:end-1) = strcat(entries(1:end-5,1:end-1),' & ');
    entries(1:2:end-5,end) = strcat(entries(1:2:end-5,end),' [-4pt] ');

    entries(end-3:end,end) = strcat(entries(end-3:end,end),' \\ ');
    entries(end-3:end,1:end-1) = strcat(entries(end-3:end,1:end-1),' & ');
    entries(end-3:2:end,end) = strcat(entries(end-3:2:end,end),' [-4pt] ');

    % Add df
    entries(end+1,:) = cellstr(['df',strcat('&',string(dfs))]);
    entries(end,end) = strcat(entries(end,end),' \\ ');

    % Footer
    footer = cell(2,ncols);
    footer{1,1} = '\bottomrule';
    footer{2,1} = '\end{tabular*}';

    % Concatenate 
    tablatex = [header;entries;footer];

    % Save as *.tex
    if save_tex
        fid = fopen(['Results/',savename,'.tex'],'w');
        [nrows,ncols] = size(tablatex);
        tabformat = [repmat('%s ',1,ncols),' \n'];
        for row = 1:nrows
            fprintf(fid,tabformat,tablatex{row,:});
        end
        fclose(fid);
    end

    % -------------------------------------------------------------------------------------
    %    Moments
    % -------------------------------------------------------------------------------------
    savename = ['extended-psi' num2str(Ppsi,'%.1f') '-moments'];
    momstrs = {'E[\Delta c]','E[\Delta d]','E[r_d]','E[r_f]','E[z_d]','E[r_{f,5}]','E[r_{f,20}]',...
        'SD[\Delta c]','SD[\Delta d]','SD[r_d]','SD[r_f]','SD[z_d]',...
        'AC[\Delta c]','AC[\Delta d]','AC[r_d]','AC[r_f]','AC[z_d]',...
        'Corr[\Delta c,\Delta d]','Corr[\Delta c,r_d]','Corr[\Delta d,r_d]',...
        'Corr[ep,z_{d,-1}]','Corr[\Delta c,z_{d,-1}]'};
    ncols = 2 + nmods;

    % Header
    header = cell(6,ncols);
    header{1,1} = '\begin{tabular*}{\textwidth}{@{\extracolsep{\fill}}l*{7}{c}}';
    header{2,1} = '\toprule';
    header{3,1} = '         & & \multicolumn{3}{c}{Omits $E[r_{f,5}]$ \& $E[r_{f,20}]$} & \multicolumn{3}{c}{All Moments}  \\[2pt]';
    header{4,1} = '             \cline{3-5}                       \cline{6-8} \rule{0pt}{\normalbaselineskip}\linebreak';
    header{5,1} = '{Moment} & {Data} & {No VR$+$SV} & {Demand} & {Demand$+$SV} & {No VR$+$SV} & {Demand} & {Demand$+$SV}   \\';
    header{6,1} = '\midrule';

    % Entries
    entries = [];
    entry = cell(1,ncols);
    for imom = 1:Fnestmom
        ismmidx = O.smmidx(imomord(imom));
        % Column 1
        entry{1,1} = ['$',momstrs{imomord(imom)},'$'];
        % Column 2: Data
        entry{1,2} = ['\num{', ... 
            num2str(table2array(tab{2,2}(ismmidx,1)),'%.2f'),'}'];
        % Columns 4+: Model means
        for imod = 1:nmods
            if ~isnan(table2array(tab{2,imod}(ismmidx,3)))
                entry{1,2+imod} = ['\num{', ... 
                    num2str(table2array(tab{2,imod}(ismmidx,3)),'%.2f'),'}'];
                entry{2,2+imod} = ['\Tiny(\num{', ... 
                    num2str(table2array(tab{2,imod}(ismmidx,4)),'%.2f'),'})'];
            else
                entry{1,2+imod} = '{$-$}';
                entry{2,2+imod} = '';
            end
        end
        % Concatenate entries
        entries = [entries;entry];
    end

    % Add line breaks, cell breaks, and spacing
    entries(:,end) = strcat(entries(:,end),' \\ ');
    entries(:,1:end-1) = strcat(entries(:,1:end-1),' & ');
    entries(1:2:end,end) = strcat(entries(1:2:end,end),' [-4pt] ');

    % Footer
    footer = cell(2,ncols);
    footer{1,1} = '\bottomrule';
    footer{2,1} = '\end{tabular*}';

    % Concatenate 
    tablatex = [header;entries;footer];

    % Save as *.tex
    if save_tex
        fid = fopen(['Results/',savename,'.tex'],'w');
        [nrows,ncols] = size(tablatex);
        tabformat = [repmat('%s ',1,ncols),' \n'];
        for row = 1:nrows
            fprintf(fid,tabformat,tablatex{row,:});
        end
        fclose(fid);
    end
end